sqlglot.dialects.postgres API documentation

Edit on GitHub

# sqlglot.dialects.postgres

View Source

      1from __future__ import annotations
      2
      3import typing as t
      4
      5from sqlglot import exp, generator, parser, tokens, transforms
      6from sqlglot.dialects.dialect import (
      7    DATE_ADD_OR_SUB,
      8    Dialect,
      9    JSON_EXTRACT_TYPE,
     10    any_value_to_max_sql,
     11    binary_from_function,
     12    bool_xor_sql,
     13    datestrtodate_sql,
     14    build_formatted_time,
     15    filter_array_using_unnest,
     16    json_extract_segments,
     17    json_path_key_only_name,
     18    max_or_greatest,
     19    merge_without_target_sql,
     20    min_or_least,
     21    no_last_day_sql,
     22    no_map_from_entries_sql,
     23    no_paren_current_date_sql,
     24    no_pivot_sql,
     25    no_trycast_sql,
     26    build_json_extract_path,
     27    build_timestamp_trunc,
     28    rename_func,
     29    sha256_sql,
     30    str_position_sql,
     31    struct_extract_sql,
     32    timestamptrunc_sql,
     33    timestrtotime_sql,
     34    trim_sql,
     35    ts_or_ds_add_cast,
     36)
     37from sqlglot.helper import is_int, seq_get
     38from sqlglot.parser import binary_range_parser
     39from sqlglot.tokens import TokenType
     40
     41DATE_DIFF_FACTOR = {
     42    "MICROSECOND": " * 1000000",
     43    "MILLISECOND": " * 1000",
     44    "SECOND": "",
     45    "MINUTE": " / 60",
     46    "HOUR": " / 3600",
     47    "DAY": " / 86400",
     48}
     49
     50
     51def _date_add_sql(kind: str) -> t.Callable[[Postgres.Generator, DATE_ADD_OR_SUB], str]:
     52    def func(self: Postgres.Generator, expression: DATE_ADD_OR_SUB) -> str:
     53        if isinstance(expression, exp.TsOrDsAdd):
     54            expression = ts_or_ds_add_cast(expression)
     55
     56        this = self.sql(expression, "this")
     57        unit = expression.args.get("unit")
     58
     59        e = self._simplify_unless_literal(expression.expression)
     60        if isinstance(e, exp.Literal):
     61            e.args["is_string"] = True
     62        elif e.is_number:
     63            e = exp.Literal.string(e.to_py())
     64        else:
     65            self.unsupported("Cannot add non literal")
     66
     67        return f"{this} {kind} {self.sql(exp.Interval(this=e, unit=unit))}"
     68
     69    return func
     70
     71
     72def _date_diff_sql(self: Postgres.Generator, expression: exp.DateDiff) -> str:
     73    unit = expression.text("unit").upper()
     74    factor = DATE_DIFF_FACTOR.get(unit)
     75
     76    end = f"CAST({self.sql(expression, 'this')} AS TIMESTAMP)"
     77    start = f"CAST({self.sql(expression, 'expression')} AS TIMESTAMP)"
     78
     79    if factor is not None:
     80        return f"CAST(EXTRACT(epoch FROM {end} - {start}){factor} AS BIGINT)"
     81
     82    age = f"AGE({end}, {start})"
     83
     84    if unit == "WEEK":
     85        unit = f"EXTRACT(days FROM ({end} - {start})) / 7"
     86    elif unit == "MONTH":
     87        unit = f"EXTRACT(year FROM {age}) * 12 + EXTRACT(month FROM {age})"
     88    elif unit == "QUARTER":
     89        unit = f"EXTRACT(year FROM {age}) * 4 + EXTRACT(month FROM {age}) / 3"
     90    elif unit == "YEAR":
     91        unit = f"EXTRACT(year FROM {age})"
     92    else:
     93        unit = age
     94
     95    return f"CAST({unit} AS BIGINT)"
     96
     97
     98def _substring_sql(self: Postgres.Generator, expression: exp.Substring) -> str:
     99    this = self.sql(expression, "this")
    100    start = self.sql(expression, "start")
    101    length = self.sql(expression, "length")
    102
    103    from_part = f" FROM {start}" if start else ""
    104    for_part = f" FOR {length}" if length else ""
    105
    106    return f"SUBSTRING({this}{from_part}{for_part})"
    107
    108
    109def _string_agg_sql(self: Postgres.Generator, expression: exp.GroupConcat) -> str:
    110    separator = expression.args.get("separator") or exp.Literal.string(",")
    111
    112    order = ""
    113    this = expression.this
    114    if isinstance(this, exp.Order):
    115        if this.this:
    116            this = this.this.pop()
    117        order = self.sql(expression.this)  # Order has a leading space
    118
    119    return f"STRING_AGG({self.format_args(this, separator)}{order})"
    120
    121
    122def _auto_increment_to_serial(expression: exp.Expression) -> exp.Expression:
    123    auto = expression.find(exp.AutoIncrementColumnConstraint)
    124
    125    if auto:
    126        expression.args["constraints"].remove(auto.parent)
    127        kind = expression.args["kind"]
    128
    129        if kind.this == exp.DataType.Type.INT:
    130            kind.replace(exp.DataType(this=exp.DataType.Type.SERIAL))
    131        elif kind.this == exp.DataType.Type.SMALLINT:
    132            kind.replace(exp.DataType(this=exp.DataType.Type.SMALLSERIAL))
    133        elif kind.this == exp.DataType.Type.BIGINT:
    134            kind.replace(exp.DataType(this=exp.DataType.Type.BIGSERIAL))
    135
    136    return expression
    137
    138
    139def _serial_to_generated(expression: exp.Expression) -> exp.Expression:
    140    if not isinstance(expression, exp.ColumnDef):
    141        return expression
    142    kind = expression.kind
    143    if not kind:
    144        return expression
    145
    146    if kind.this == exp.DataType.Type.SERIAL:
    147        data_type = exp.DataType(this=exp.DataType.Type.INT)
    148    elif kind.this == exp.DataType.Type.SMALLSERIAL:
    149        data_type = exp.DataType(this=exp.DataType.Type.SMALLINT)
    150    elif kind.this == exp.DataType.Type.BIGSERIAL:
    151        data_type = exp.DataType(this=exp.DataType.Type.BIGINT)
    152    else:
    153        data_type = None
    154
    155    if data_type:
    156        expression.args["kind"].replace(data_type)
    157        constraints = expression.args["constraints"]
    158        generated = exp.ColumnConstraint(kind=exp.GeneratedAsIdentityColumnConstraint(this=False))
    159        notnull = exp.ColumnConstraint(kind=exp.NotNullColumnConstraint())
    160
    161        if notnull not in constraints:
    162            constraints.insert(0, notnull)
    163        if generated not in constraints:
    164            constraints.insert(0, generated)
    165
    166    return expression
    167
    168
    169def _build_generate_series(args: t.List) -> exp.GenerateSeries:
    170    # The goal is to convert step values like '1 day' or INTERVAL '1 day' into INTERVAL '1' day
    171    step = seq_get(args, 2)
    172
    173    if step is None:
    174        # Postgres allows calls with just two arguments -- the "step" argument defaults to 1
    175        return exp.GenerateSeries.from_arg_list(args)
    176
    177    if step.is_string:
    178        args[2] = exp.to_interval(step.this)
    179    elif isinstance(step, exp.Interval) and not step.args.get("unit"):
    180        args[2] = exp.to_interval(step.this.this)
    181
    182    return exp.GenerateSeries.from_arg_list(args)
    183
    184
    185def _build_to_timestamp(args: t.List) -> exp.UnixToTime | exp.StrToTime:
    186    # TO_TIMESTAMP accepts either a single double argument or (text, text)
    187    if len(args) == 1:
    188        # https://www.postgresql.org/docs/current/functions-datetime.html#FUNCTIONS-DATETIME-TABLE
    189        return exp.UnixToTime.from_arg_list(args)
    190
    191    # https://www.postgresql.org/docs/current/functions-formatting.html
    192    return build_formatted_time(exp.StrToTime, "postgres")(args)
    193
    194
    195def _json_extract_sql(
    196    name: str, op: str
    197) -> t.Callable[[Postgres.Generator, JSON_EXTRACT_TYPE], str]:
    198    def _generate(self: Postgres.Generator, expression: JSON_EXTRACT_TYPE) -> str:
    199        if expression.args.get("only_json_types"):
    200            return json_extract_segments(name, quoted_index=False, op=op)(self, expression)
    201        return json_extract_segments(name)(self, expression)
    202
    203    return _generate
    204
    205
    206def _build_regexp_replace(args: t.List) -> exp.RegexpReplace:
    207    # The signature of REGEXP_REPLACE is:
    208    # regexp_replace(source, pattern, replacement [, start [, N ]] [, flags ])
    209    #
    210    # Any one of `start`, `N` and `flags` can be column references, meaning that
    211    # unless we can statically see that the last argument is a non-integer string
    212    # (eg. not '0'), then it's not possible to construct the correct AST
    213    if len(args) > 3:
    214        last = args[-1]
    215        if not is_int(last.name):
    216            if not last.type or last.is_type(exp.DataType.Type.UNKNOWN, exp.DataType.Type.NULL):
    217                from sqlglot.optimizer.annotate_types import annotate_types
    218
    219                last = annotate_types(last)
    220
    221            if last.is_type(*exp.DataType.TEXT_TYPES):
    222                regexp_replace = exp.RegexpReplace.from_arg_list(args[:-1])
    223                regexp_replace.set("modifiers", last)
    224                return regexp_replace
    225
    226    return exp.RegexpReplace.from_arg_list(args)
    227
    228
    229def _unix_to_time_sql(self: Postgres.Generator, expression: exp.UnixToTime) -> str:
    230    scale = expression.args.get("scale")
    231    timestamp = expression.this
    232
    233    if scale in (None, exp.UnixToTime.SECONDS):
    234        return self.func("TO_TIMESTAMP", timestamp, self.format_time(expression))
    235
    236    return self.func(
    237        "TO_TIMESTAMP",
    238        exp.Div(this=timestamp, expression=exp.func("POW", 10, scale)),
    239        self.format_time(expression),
    240    )
    241
    242
    243class Postgres(Dialect):
    244    INDEX_OFFSET = 1
    245    TYPED_DIVISION = True
    246    CONCAT_COALESCE = True
    247    NULL_ORDERING = "nulls_are_large"
    248    TIME_FORMAT = "'YYYY-MM-DD HH24:MI:SS'"
    249    TABLESAMPLE_SIZE_IS_PERCENT = True
    250
    251    TIME_MAPPING = {
    252        "AM": "%p",
    253        "PM": "%p",
    254        "D": "%u",  # 1-based day of week
    255        "DD": "%d",  # day of month
    256        "DDD": "%j",  # zero padded day of year
    257        "FMDD": "%-d",  # - is no leading zero for Python; same for FM in postgres
    258        "FMDDD": "%-j",  # day of year
    259        "FMHH12": "%-I",  # 9
    260        "FMHH24": "%-H",  # 9
    261        "FMMI": "%-M",  # Minute
    262        "FMMM": "%-m",  # 1
    263        "FMSS": "%-S",  # Second
    264        "HH12": "%I",  # 09
    265        "HH24": "%H",  # 09
    266        "MI": "%M",  # zero padded minute
    267        "MM": "%m",  # 01
    268        "OF": "%z",  # utc offset
    269        "SS": "%S",  # zero padded second
    270        "TMDay": "%A",  # TM is locale dependent
    271        "TMDy": "%a",
    272        "TMMon": "%b",  # Sep
    273        "TMMonth": "%B",  # September
    274        "TZ": "%Z",  # uppercase timezone name
    275        "US": "%f",  # zero padded microsecond
    276        "WW": "%U",  # 1-based week of year
    277        "YY": "%y",  # 15
    278        "YYYY": "%Y",  # 2015
    279    }
    280
    281    class Tokenizer(tokens.Tokenizer):
    282        BIT_STRINGS = [("b'", "'"), ("B'", "'")]
    283        HEX_STRINGS = [("x'", "'"), ("X'", "'")]
    284        BYTE_STRINGS = [("e'", "'"), ("E'", "'")]
    285        HEREDOC_STRINGS = ["$"]
    286
    287        HEREDOC_TAG_IS_IDENTIFIER = True
    288        HEREDOC_STRING_ALTERNATIVE = TokenType.PARAMETER
    289
    290        KEYWORDS = {
    291            **tokens.Tokenizer.KEYWORDS,
    292            "~~": TokenType.LIKE,
    293            "~~*": TokenType.ILIKE,
    294            "~*": TokenType.IRLIKE,
    295            "~": TokenType.RLIKE,
    296            "@@": TokenType.DAT,
    297            "@>": TokenType.AT_GT,
    298            "<@": TokenType.LT_AT,
    299            "|/": TokenType.PIPE_SLASH,
    300            "||/": TokenType.DPIPE_SLASH,
    301            "BEGIN": TokenType.COMMAND,
    302            "BEGIN TRANSACTION": TokenType.BEGIN,
    303            "BIGSERIAL": TokenType.BIGSERIAL,
    304            "CHARACTER VARYING": TokenType.VARCHAR,
    305            "CONSTRAINT TRIGGER": TokenType.COMMAND,
    306            "CSTRING": TokenType.PSEUDO_TYPE,
    307            "DECLARE": TokenType.COMMAND,
    308            "DO": TokenType.COMMAND,
    309            "EXEC": TokenType.COMMAND,
    310            "HSTORE": TokenType.HSTORE,
    311            "INT8": TokenType.BIGINT,
    312            "MONEY": TokenType.MONEY,
    313            "NAME": TokenType.NAME,
    314            "OID": TokenType.OBJECT_IDENTIFIER,
    315            "ONLY": TokenType.ONLY,
    316            "OPERATOR": TokenType.OPERATOR,
    317            "REFRESH": TokenType.COMMAND,
    318            "REINDEX": TokenType.COMMAND,
    319            "RESET": TokenType.COMMAND,
    320            "REVOKE": TokenType.COMMAND,
    321            "SERIAL": TokenType.SERIAL,
    322            "SMALLSERIAL": TokenType.SMALLSERIAL,
    323            "TEMP": TokenType.TEMPORARY,
    324            "REGCLASS": TokenType.OBJECT_IDENTIFIER,
    325            "REGCOLLATION": TokenType.OBJECT_IDENTIFIER,
    326            "REGCONFIG": TokenType.OBJECT_IDENTIFIER,
    327            "REGDICTIONARY": TokenType.OBJECT_IDENTIFIER,
    328            "REGNAMESPACE": TokenType.OBJECT_IDENTIFIER,
    329            "REGOPER": TokenType.OBJECT_IDENTIFIER,
    330            "REGOPERATOR": TokenType.OBJECT_IDENTIFIER,
    331            "REGPROC": TokenType.OBJECT_IDENTIFIER,
    332            "REGPROCEDURE": TokenType.OBJECT_IDENTIFIER,
    333            "REGROLE": TokenType.OBJECT_IDENTIFIER,
    334            "REGTYPE": TokenType.OBJECT_IDENTIFIER,
    335            "FLOAT": TokenType.DOUBLE,
    336        }
    337        KEYWORDS.pop("/*+")
    338        KEYWORDS.pop("DIV")
    339
    340        SINGLE_TOKENS = {
    341            **tokens.Tokenizer.SINGLE_TOKENS,
    342            "$": TokenType.HEREDOC_STRING,
    343        }
    344
    345        VAR_SINGLE_TOKENS = {"$"}
    346
    347    class Parser(parser.Parser):
    348        PROPERTY_PARSERS = {
    349            **parser.Parser.PROPERTY_PARSERS,
    350            "SET": lambda self: self.expression(exp.SetConfigProperty, this=self._parse_set()),
    351        }
    352        PROPERTY_PARSERS.pop("INPUT")
    353
    354        FUNCTIONS = {
    355            **parser.Parser.FUNCTIONS,
    356            "DATE_TRUNC": build_timestamp_trunc,
    357            "DIV": lambda args: exp.cast(
    358                binary_from_function(exp.IntDiv)(args), exp.DataType.Type.DECIMAL
    359            ),
    360            "GENERATE_SERIES": _build_generate_series,
    361            "JSON_EXTRACT_PATH": build_json_extract_path(exp.JSONExtract),
    362            "JSON_EXTRACT_PATH_TEXT": build_json_extract_path(exp.JSONExtractScalar),
    363            "MAKE_TIME": exp.TimeFromParts.from_arg_list,
    364            "MAKE_TIMESTAMP": exp.TimestampFromParts.from_arg_list,
    365            "NOW": exp.CurrentTimestamp.from_arg_list,
    366            "REGEXP_REPLACE": _build_regexp_replace,
    367            "TO_CHAR": build_formatted_time(exp.TimeToStr, "postgres"),
    368            "TO_DATE": build_formatted_time(exp.StrToDate, "postgres"),
    369            "TO_TIMESTAMP": _build_to_timestamp,
    370            "UNNEST": exp.Explode.from_arg_list,
    371            "SHA256": lambda args: exp.SHA2(this=seq_get(args, 0), length=exp.Literal.number(256)),
    372            "SHA384": lambda args: exp.SHA2(this=seq_get(args, 0), length=exp.Literal.number(384)),
    373            "SHA512": lambda args: exp.SHA2(this=seq_get(args, 0), length=exp.Literal.number(512)),
    374        }
    375
    376        FUNCTION_PARSERS = {
    377            **parser.Parser.FUNCTION_PARSERS,
    378            "DATE_PART": lambda self: self._parse_date_part(),
    379        }
    380
    381        BITWISE = {
    382            **parser.Parser.BITWISE,
    383            TokenType.HASH: exp.BitwiseXor,
    384        }
    385
    386        EXPONENT = {
    387            TokenType.CARET: exp.Pow,
    388        }
    389
    390        RANGE_PARSERS = {
    391            **parser.Parser.RANGE_PARSERS,
    392            TokenType.AT_GT: binary_range_parser(exp.ArrayContainsAll),
    393            TokenType.DAMP: binary_range_parser(exp.ArrayOverlaps),
    394            TokenType.DAT: lambda self, this: self.expression(
    395                exp.MatchAgainst, this=self._parse_bitwise(), expressions=[this]
    396            ),
    397            TokenType.LT_AT: binary_range_parser(exp.ArrayContainsAll, reverse_args=True),
    398            TokenType.OPERATOR: lambda self, this: self._parse_operator(this),
    399        }
    400
    401        STATEMENT_PARSERS = {
    402            **parser.Parser.STATEMENT_PARSERS,
    403            TokenType.END: lambda self: self._parse_commit_or_rollback(),
    404        }
    405
    406        JSON_ARROWS_REQUIRE_JSON_TYPE = True
    407
    408        COLUMN_OPERATORS = {
    409            **parser.Parser.COLUMN_OPERATORS,
    410            TokenType.ARROW: lambda self, this, path: build_json_extract_path(
    411                exp.JSONExtract, arrow_req_json_type=self.JSON_ARROWS_REQUIRE_JSON_TYPE
    412            )([this, path]),
    413            TokenType.DARROW: lambda self, this, path: build_json_extract_path(
    414                exp.JSONExtractScalar, arrow_req_json_type=self.JSON_ARROWS_REQUIRE_JSON_TYPE
    415            )([this, path]),
    416        }
    417
    418        def _parse_operator(self, this: t.Optional[exp.Expression]) -> t.Optional[exp.Expression]:
    419            while True:
    420                if not self._match(TokenType.L_PAREN):
    421                    break
    422
    423                op = ""
    424                while self._curr and not self._match(TokenType.R_PAREN):
    425                    op += self._curr.text
    426                    self._advance()
    427
    428                this = self.expression(
    429                    exp.Operator,
    430                    comments=self._prev_comments,
    431                    this=this,
    432                    operator=op,
    433                    expression=self._parse_bitwise(),
    434                )
    435
    436                if not self._match(TokenType.OPERATOR):
    437                    break
    438
    439            return this
    440
    441        def _parse_date_part(self) -> exp.Expression:
    442            part = self._parse_type()
    443            self._match(TokenType.COMMA)
    444            value = self._parse_bitwise()
    445
    446            if part and part.is_string:
    447                part = exp.var(part.name)
    448
    449            return self.expression(exp.Extract, this=part, expression=value)
    450
    451        def _parse_unique_key(self) -> t.Optional[exp.Expression]:
    452            return None
    453
    454    class Generator(generator.Generator):
    455        SINGLE_STRING_INTERVAL = True
    456        RENAME_TABLE_WITH_DB = False
    457        LOCKING_READS_SUPPORTED = True
    458        JOIN_HINTS = False
    459        TABLE_HINTS = False
    460        QUERY_HINTS = False
    461        NVL2_SUPPORTED = False
    462        PARAMETER_TOKEN = "$"
    463        TABLESAMPLE_SIZE_IS_ROWS = False
    464        TABLESAMPLE_SEED_KEYWORD = "REPEATABLE"
    465        SUPPORTS_SELECT_INTO = True
    466        JSON_TYPE_REQUIRED_FOR_EXTRACTION = True
    467        SUPPORTS_UNLOGGED_TABLES = True
    468        LIKE_PROPERTY_INSIDE_SCHEMA = True
    469        MULTI_ARG_DISTINCT = False
    470        CAN_IMPLEMENT_ARRAY_ANY = True
    471        COPY_HAS_INTO_KEYWORD = False
    472
    473        SUPPORTED_JSON_PATH_PARTS = {
    474            exp.JSONPathKey,
    475            exp.JSONPathRoot,
    476            exp.JSONPathSubscript,
    477        }
    478
    479        TYPE_MAPPING = {
    480            **generator.Generator.TYPE_MAPPING,
    481            exp.DataType.Type.TINYINT: "SMALLINT",
    482            exp.DataType.Type.FLOAT: "REAL",
    483            exp.DataType.Type.DOUBLE: "DOUBLE PRECISION",
    484            exp.DataType.Type.BINARY: "BYTEA",
    485            exp.DataType.Type.VARBINARY: "BYTEA",
    486            exp.DataType.Type.ROWVERSION: "BYTEA",
    487            exp.DataType.Type.DATETIME: "TIMESTAMP",
    488        }
    489
    490        TRANSFORMS = {
    491            **generator.Generator.TRANSFORMS,
    492            exp.AnyValue: any_value_to_max_sql,
    493            exp.Array: lambda self, e: (
    494                f"{self.normalize_func('ARRAY')}({self.sql(e.expressions[0])})"
    495                if isinstance(seq_get(e.expressions, 0), exp.Select)
    496                else f"{self.normalize_func('ARRAY')}[{self.expressions(e, flat=True)}]"
    497            ),
    498            exp.ArrayConcat: rename_func("ARRAY_CAT"),
    499            exp.ArrayContainsAll: lambda self, e: self.binary(e, "@>"),
    500            exp.ArrayOverlaps: lambda self, e: self.binary(e, "&&"),
    501            exp.ArrayFilter: filter_array_using_unnest,
    502            exp.ArraySize: lambda self, e: self.func("ARRAY_LENGTH", e.this, e.expression or "1"),
    503            exp.BitwiseXor: lambda self, e: self.binary(e, "#"),
    504            exp.ColumnDef: transforms.preprocess([_auto_increment_to_serial, _serial_to_generated]),
    505            exp.CurrentDate: no_paren_current_date_sql,
    506            exp.CurrentTimestamp: lambda *_: "CURRENT_TIMESTAMP",
    507            exp.CurrentUser: lambda *_: "CURRENT_USER",
    508            exp.DateAdd: _date_add_sql("+"),
    509            exp.DateDiff: _date_diff_sql,
    510            exp.DateStrToDate: datestrtodate_sql,
    511            exp.DateSub: _date_add_sql("-"),
    512            exp.Explode: rename_func("UNNEST"),
    513            exp.GroupConcat: _string_agg_sql,
    514            exp.IntDiv: rename_func("DIV"),
    515            exp.JSONExtract: _json_extract_sql("JSON_EXTRACT_PATH", "->"),
    516            exp.JSONExtractScalar: _json_extract_sql("JSON_EXTRACT_PATH_TEXT", "->>"),
    517            exp.JSONBExtract: lambda self, e: self.binary(e, "#>"),
    518            exp.JSONBExtractScalar: lambda self, e: self.binary(e, "#>>"),
    519            exp.JSONBContains: lambda self, e: self.binary(e, "?"),
    520            exp.ParseJSON: lambda self, e: self.sql(exp.cast(e.this, exp.DataType.Type.JSON)),
    521            exp.JSONPathKey: json_path_key_only_name,
    522            exp.JSONPathRoot: lambda *_: "",
    523            exp.JSONPathSubscript: lambda self, e: self.json_path_part(e.this),
    524            exp.LastDay: no_last_day_sql,
    525            exp.LogicalOr: rename_func("BOOL_OR"),
    526            exp.LogicalAnd: rename_func("BOOL_AND"),
    527            exp.Max: max_or_greatest,
    528            exp.MapFromEntries: no_map_from_entries_sql,
    529            exp.Min: min_or_least,
    530            exp.Merge: merge_without_target_sql,
    531            exp.PartitionedByProperty: lambda self, e: f"PARTITION BY {self.sql(e, 'this')}",
    532            exp.PercentileCont: transforms.preprocess(
    533                [transforms.add_within_group_for_percentiles]
    534            ),
    535            exp.PercentileDisc: transforms.preprocess(
    536                [transforms.add_within_group_for_percentiles]
    537            ),
    538            exp.Pivot: no_pivot_sql,
    539            exp.Pow: lambda self, e: self.binary(e, "^"),
    540            exp.Rand: rename_func("RANDOM"),
    541            exp.RegexpLike: lambda self, e: self.binary(e, "~"),
    542            exp.RegexpILike: lambda self, e: self.binary(e, "~*"),
    543            exp.Select: transforms.preprocess(
    544                [
    545                    transforms.eliminate_semi_and_anti_joins,
    546                    transforms.eliminate_qualify,
    547                ]
    548            ),
    549            exp.SHA2: sha256_sql,
    550            exp.StrPosition: str_position_sql,
    551            exp.StrToDate: lambda self, e: self.func("TO_DATE", e.this, self.format_time(e)),
    552            exp.StrToTime: lambda self, e: self.func("TO_TIMESTAMP", e.this, self.format_time(e)),
    553            exp.StructExtract: struct_extract_sql,
    554            exp.Substring: _substring_sql,
    555            exp.TimeFromParts: rename_func("MAKE_TIME"),
    556            exp.TimestampFromParts: rename_func("MAKE_TIMESTAMP"),
    557            exp.TimestampTrunc: timestamptrunc_sql(zone=True),
    558            exp.TimeStrToTime: timestrtotime_sql,
    559            exp.TimeToStr: lambda self, e: self.func("TO_CHAR", e.this, self.format_time(e)),
    560            exp.ToChar: lambda self, e: self.function_fallback_sql(e),
    561            exp.Trim: trim_sql,
    562            exp.TryCast: no_trycast_sql,
    563            exp.TsOrDsAdd: _date_add_sql("+"),
    564            exp.TsOrDsDiff: _date_diff_sql,
    565            exp.UnixToTime: lambda self, e: self.func("TO_TIMESTAMP", e.this),
    566            exp.TimeToUnix: lambda self, e: self.func(
    567                "DATE_PART", exp.Literal.string("epoch"), e.this
    568            ),
    569            exp.VariancePop: rename_func("VAR_POP"),
    570            exp.Variance: rename_func("VAR_SAMP"),
    571            exp.Xor: bool_xor_sql,
    572            exp.UnixToTime: _unix_to_time_sql,
    573        }
    574        TRANSFORMS.pop(exp.CommentColumnConstraint)
    575
    576        PROPERTIES_LOCATION = {
    577            **generator.Generator.PROPERTIES_LOCATION,
    578            exp.PartitionedByProperty: exp.Properties.Location.POST_SCHEMA,
    579            exp.TransientProperty: exp.Properties.Location.UNSUPPORTED,
    580            exp.VolatileProperty: exp.Properties.Location.UNSUPPORTED,
    581        }
    582
    583        def schemacommentproperty_sql(self, expression: exp.SchemaCommentProperty) -> str:
    584            self.unsupported("Table comments are not supported in the CREATE statement")
    585            return ""
    586
    587        def commentcolumnconstraint_sql(self, expression: exp.CommentColumnConstraint) -> str:
    588            self.unsupported("Column comments are not supported in the CREATE statement")
    589            return ""
    590
    591        def unnest_sql(self, expression: exp.Unnest) -> str:
    592            if len(expression.expressions) == 1:
    593                from sqlglot.optimizer.annotate_types import annotate_types
    594
    595                this = annotate_types(expression.expressions[0])
    596                if this.is_type("array<json>"):
    597                    while isinstance(this, exp.Cast):
    598                        this = this.this
    599
    600                    arg = self.sql(exp.cast(this, exp.DataType.Type.JSON))
    601                    alias = self.sql(expression, "alias")
    602                    alias = f" AS {alias}" if alias else ""
    603
    604                    if expression.args.get("offset"):
    605                        self.unsupported("Unsupported JSON_ARRAY_ELEMENTS with offset")
    606
    607                    return f"JSON_ARRAY_ELEMENTS({arg}){alias}"
    608
    609            return super().unnest_sql(expression)
    610
    611        def bracket_sql(self, expression: exp.Bracket) -> str:
    612            """Forms like ARRAY[1, 2, 3][3] aren't allowed; we need to wrap the ARRAY."""
    613            if isinstance(expression.this, exp.Array):
    614                expression.set("this", exp.paren(expression.this, copy=False))
    615
    616            return super().bracket_sql(expression)
    617
    618        def matchagainst_sql(self, expression: exp.MatchAgainst) -> str:
    619            this = self.sql(expression, "this")
    620            expressions = [f"{self.sql(e)} @@ {this}" for e in expression.expressions]
    621            sql = " OR ".join(expressions)
    622            return f"({sql})" if len(expressions) > 1 else sql
    623
    624        def alterset_sql(self, expression: exp.AlterSet) -> str:
    625            exprs = self.expressions(expression, flat=True)
    626            exprs = f"({exprs})" if exprs else ""
    627
    628            access_method = self.sql(expression, "access_method")
    629            access_method = f"ACCESS METHOD {access_method}" if access_method else ""
    630            tablespace = self.sql(expression, "tablespace")
    631            tablespace = f"TABLESPACE {tablespace}" if tablespace else ""
    632            option = self.sql(expression, "option")
    633
    634            return f"SET {exprs}{access_method}{tablespace}{option}"
    635
    636        def datatype_sql(self, expression: exp.DataType) -> str:
    637            if expression.is_type(exp.DataType.Type.ARRAY):
    638                if expression.expressions:
    639                    values = self.expressions(expression, key="values", flat=True)
    640                    return f"{self.expressions(expression, flat=True)}[{values}]"
    641                return "ARRAY"
    642            return super().datatype_sql(expression)
    643
    644        def cast_sql(self, expression: exp.Cast, safe_prefix: t.Optional[str] = None) -> str:
    645            this = expression.this
    646
    647            # Postgres casts DIV() to decimal for transpilation but when roundtripping it's superfluous
    648            if isinstance(this, exp.IntDiv) and expression.to == exp.DataType.build("decimal"):
    649                return self.sql(this)
    650
    651            return super().cast_sql(expression, safe_prefix=safe_prefix)

DATE_DIFF_FACTOR = {'MICROSECOND': ' * 1000000', 'MILLISECOND': ' * 1000', 'SECOND': '', 'MINUTE': ' / 60', 'HOUR': ' / 3600', 'DAY': ' / 86400'}
class Postgres(sqlglot.dialects.dialect.Dialect): View Source

    244class Postgres(Dialect):
    245    INDEX_OFFSET = 1
    246    TYPED_DIVISION = True
    247    CONCAT_COALESCE = True
    248    NULL_ORDERING = "nulls_are_large"
    249    TIME_FORMAT = "'YYYY-MM-DD HH24:MI:SS'"
    250    TABLESAMPLE_SIZE_IS_PERCENT = True
    251
    252    TIME_MAPPING = {
    253        "AM": "%p",
    254        "PM": "%p",
    255        "D": "%u",  # 1-based day of week
    256        "DD": "%d",  # day of month
    257        "DDD": "%j",  # zero padded day of year
    258        "FMDD": "%-d",  # - is no leading zero for Python; same for FM in postgres
    259        "FMDDD": "%-j",  # day of year
    260        "FMHH12": "%-I",  # 9
    261        "FMHH24": "%-H",  # 9
    262        "FMMI": "%-M",  # Minute
    263        "FMMM": "%-m",  # 1
    264        "FMSS": "%-S",  # Second
    265        "HH12": "%I",  # 09
    266        "HH24": "%H",  # 09
    267        "MI": "%M",  # zero padded minute
    268        "MM": "%m",  # 01
    269        "OF": "%z",  # utc offset
    270        "SS": "%S",  # zero padded second
    271        "TMDay": "%A",  # TM is locale dependent
    272        "TMDy": "%a",
    273        "TMMon": "%b",  # Sep
    274        "TMMonth": "%B",  # September
    275        "TZ": "%Z",  # uppercase timezone name
    276        "US": "%f",  # zero padded microsecond
    277        "WW": "%U",  # 1-based week of year
    278        "YY": "%y",  # 15
    279        "YYYY": "%Y",  # 2015
    280    }
    281
    282    class Tokenizer(tokens.Tokenizer):
    283        BIT_STRINGS = [("b'", "'"), ("B'", "'")]
    284        HEX_STRINGS = [("x'", "'"), ("X'", "'")]
    285        BYTE_STRINGS = [("e'", "'"), ("E'", "'")]
    286        HEREDOC_STRINGS = ["$"]
    287
    288        HEREDOC_TAG_IS_IDENTIFIER = True
    289        HEREDOC_STRING_ALTERNATIVE = TokenType.PARAMETER
    290
    291        KEYWORDS = {
    292            **tokens.Tokenizer.KEYWORDS,
    293            "~~": TokenType.LIKE,
    294            "~~*": TokenType.ILIKE,
    295            "~*": TokenType.IRLIKE,
    296            "~": TokenType.RLIKE,
    297            "@@": TokenType.DAT,
    298            "@>": TokenType.AT_GT,
    299            "<@": TokenType.LT_AT,
    300            "|/": TokenType.PIPE_SLASH,
    301            "||/": TokenType.DPIPE_SLASH,
    302            "BEGIN": TokenType.COMMAND,
    303            "BEGIN TRANSACTION": TokenType.BEGIN,
    304            "BIGSERIAL": TokenType.BIGSERIAL,
    305            "CHARACTER VARYING": TokenType.VARCHAR,
    306            "CONSTRAINT TRIGGER": TokenType.COMMAND,
    307            "CSTRING": TokenType.PSEUDO_TYPE,
    308            "DECLARE": TokenType.COMMAND,
    309            "DO": TokenType.COMMAND,
    310            "EXEC": TokenType.COMMAND,
    311            "HSTORE": TokenType.HSTORE,
    312            "INT8": TokenType.BIGINT,
    313            "MONEY": TokenType.MONEY,
    314            "NAME": TokenType.NAME,
    315            "OID": TokenType.OBJECT_IDENTIFIER,
    316            "ONLY": TokenType.ONLY,
    317            "OPERATOR": TokenType.OPERATOR,
    318            "REFRESH": TokenType.COMMAND,
    319            "REINDEX": TokenType.COMMAND,
    320            "RESET": TokenType.COMMAND,
    321            "REVOKE": TokenType.COMMAND,
    322            "SERIAL": TokenType.SERIAL,
    323            "SMALLSERIAL": TokenType.SMALLSERIAL,
    324            "TEMP": TokenType.TEMPORARY,
    325            "REGCLASS": TokenType.OBJECT_IDENTIFIER,
    326            "REGCOLLATION": TokenType.OBJECT_IDENTIFIER,
    327            "REGCONFIG": TokenType.OBJECT_IDENTIFIER,
    328            "REGDICTIONARY": TokenType.OBJECT_IDENTIFIER,
    329            "REGNAMESPACE": TokenType.OBJECT_IDENTIFIER,
    330            "REGOPER": TokenType.OBJECT_IDENTIFIER,
    331            "REGOPERATOR": TokenType.OBJECT_IDENTIFIER,
    332            "REGPROC": TokenType.OBJECT_IDENTIFIER,
    333            "REGPROCEDURE": TokenType.OBJECT_IDENTIFIER,
    334            "REGROLE": TokenType.OBJECT_IDENTIFIER,
    335            "REGTYPE": TokenType.OBJECT_IDENTIFIER,
    336            "FLOAT": TokenType.DOUBLE,
    337        }
    338        KEYWORDS.pop("/*+")
    339        KEYWORDS.pop("DIV")
    340
    341        SINGLE_TOKENS = {
    342            **tokens.Tokenizer.SINGLE_TOKENS,
    343            "$": TokenType.HEREDOC_STRING,
    344        }
    345
    346        VAR_SINGLE_TOKENS = {"$"}
    347
    348    class Parser(parser.Parser):
    349        PROPERTY_PARSERS = {
    350            **parser.Parser.PROPERTY_PARSERS,
    351            "SET": lambda self: self.expression(exp.SetConfigProperty, this=self._parse_set()),
    352        }
    353        PROPERTY_PARSERS.pop("INPUT")
    354
    355        FUNCTIONS = {
    356            **parser.Parser.FUNCTIONS,
    357            "DATE_TRUNC": build_timestamp_trunc,
    358            "DIV": lambda args: exp.cast(
    359                binary_from_function(exp.IntDiv)(args), exp.DataType.Type.DECIMAL
    360            ),
    361            "GENERATE_SERIES": _build_generate_series,
    362            "JSON_EXTRACT_PATH": build_json_extract_path(exp.JSONExtract),
    363            "JSON_EXTRACT_PATH_TEXT": build_json_extract_path(exp.JSONExtractScalar),
    364            "MAKE_TIME": exp.TimeFromParts.from_arg_list,
    365            "MAKE_TIMESTAMP": exp.TimestampFromParts.from_arg_list,
    366            "NOW": exp.CurrentTimestamp.from_arg_list,
    367            "REGEXP_REPLACE": _build_regexp_replace,
    368            "TO_CHAR": build_formatted_time(exp.TimeToStr, "postgres"),
    369            "TO_DATE": build_formatted_time(exp.StrToDate, "postgres"),
    370            "TO_TIMESTAMP": _build_to_timestamp,
    371            "UNNEST": exp.Explode.from_arg_list,
    372            "SHA256": lambda args: exp.SHA2(this=seq_get(args, 0), length=exp.Literal.number(256)),
    373            "SHA384": lambda args: exp.SHA2(this=seq_get(args, 0), length=exp.Literal.number(384)),
    374            "SHA512": lambda args: exp.SHA2(this=seq_get(args, 0), length=exp.Literal.number(512)),
    375        }
    376
    377        FUNCTION_PARSERS = {
    378            **parser.Parser.FUNCTION_PARSERS,
    379            "DATE_PART": lambda self: self._parse_date_part(),
    380        }
    381
    382        BITWISE = {
    383            **parser.Parser.BITWISE,
    384            TokenType.HASH: exp.BitwiseXor,
    385        }
    386
    387        EXPONENT = {
    388            TokenType.CARET: exp.Pow,
    389        }
    390
    391        RANGE_PARSERS = {
    392            **parser.Parser.RANGE_PARSERS,
    393            TokenType.AT_GT: binary_range_parser(exp.ArrayContainsAll),
    394            TokenType.DAMP: binary_range_parser(exp.ArrayOverlaps),
    395            TokenType.DAT: lambda self, this: self.expression(
    396                exp.MatchAgainst, this=self._parse_bitwise(), expressions=[this]
    397            ),
    398            TokenType.LT_AT: binary_range_parser(exp.ArrayContainsAll, reverse_args=True),
    399            TokenType.OPERATOR: lambda self, this: self._parse_operator(this),
    400        }
    401
    402        STATEMENT_PARSERS = {
    403            **parser.Parser.STATEMENT_PARSERS,
    404            TokenType.END: lambda self: self._parse_commit_or_rollback(),
    405        }
    406
    407        JSON_ARROWS_REQUIRE_JSON_TYPE = True
    408
    409        COLUMN_OPERATORS = {
    410            **parser.Parser.COLUMN_OPERATORS,
    411            TokenType.ARROW: lambda self, this, path: build_json_extract_path(
    412                exp.JSONExtract, arrow_req_json_type=self.JSON_ARROWS_REQUIRE_JSON_TYPE
    413            )([this, path]),
    414            TokenType.DARROW: lambda self, this, path: build_json_extract_path(
    415                exp.JSONExtractScalar, arrow_req_json_type=self.JSON_ARROWS_REQUIRE_JSON_TYPE
    416            )([this, path]),
    417        }
    418
    419        def _parse_operator(self, this: t.Optional[exp.Expression]) -> t.Optional[exp.Expression]:
    420            while True:
    421                if not self._match(TokenType.L_PAREN):
    422                    break
    423
    424                op = ""
    425                while self._curr and not self._match(TokenType.R_PAREN):
    426                    op += self._curr.text
    427                    self._advance()
    428
    429                this = self.expression(
    430                    exp.Operator,
    431                    comments=self._prev_comments,
    432                    this=this,
    433                    operator=op,
    434                    expression=self._parse_bitwise(),
    435                )
    436
    437                if not self._match(TokenType.OPERATOR):
    438                    break
    439
    440            return this
    441
    442        def _parse_date_part(self) -> exp.Expression:
    443            part = self._parse_type()
    444            self._match(TokenType.COMMA)
    445            value = self._parse_bitwise()
    446
    447            if part and part.is_string:
    448                part = exp.var(part.name)
    449
    450            return self.expression(exp.Extract, this=part, expression=value)
    451
    452        def _parse_unique_key(self) -> t.Optional[exp.Expression]:
    453            return None
    454
    455    class Generator(generator.Generator):
    456        SINGLE_STRING_INTERVAL = True
    457        RENAME_TABLE_WITH_DB = False
    458        LOCKING_READS_SUPPORTED = True
    459        JOIN_HINTS = False
    460        TABLE_HINTS = False
    461        QUERY_HINTS = False
    462        NVL2_SUPPORTED = False
    463        PARAMETER_TOKEN = "$"
    464        TABLESAMPLE_SIZE_IS_ROWS = False
    465        TABLESAMPLE_SEED_KEYWORD = "REPEATABLE"
    466        SUPPORTS_SELECT_INTO = True
    467        JSON_TYPE_REQUIRED_FOR_EXTRACTION = True
    468        SUPPORTS_UNLOGGED_TABLES = True
    469        LIKE_PROPERTY_INSIDE_SCHEMA = True
    470        MULTI_ARG_DISTINCT = False
    471        CAN_IMPLEMENT_ARRAY_ANY = True
    472        COPY_HAS_INTO_KEYWORD = False
    473
    474        SUPPORTED_JSON_PATH_PARTS = {
    475            exp.JSONPathKey,
    476            exp.JSONPathRoot,
    477            exp.JSONPathSubscript,
    478        }
    479
    480        TYPE_MAPPING = {
    481            **generator.Generator.TYPE_MAPPING,
    482            exp.DataType.Type.TINYINT: "SMALLINT",
    483            exp.DataType.Type.FLOAT: "REAL",
    484            exp.DataType.Type.DOUBLE: "DOUBLE PRECISION",
    485            exp.DataType.Type.BINARY: "BYTEA",
    486            exp.DataType.Type.VARBINARY: "BYTEA",
    487            exp.DataType.Type.ROWVERSION: "BYTEA",
    488            exp.DataType.Type.DATETIME: "TIMESTAMP",
    489        }
    490
    491        TRANSFORMS = {
    492            **generator.Generator.TRANSFORMS,
    493            exp.AnyValue: any_value_to_max_sql,
    494            exp.Array: lambda self, e: (
    495                f"{self.normalize_func('ARRAY')}({self.sql(e.expressions[0])})"
    496                if isinstance(seq_get(e.expressions, 0), exp.Select)
    497                else f"{self.normalize_func('ARRAY')}[{self.expressions(e, flat=True)}]"
    498            ),
    499            exp.ArrayConcat: rename_func("ARRAY_CAT"),
    500            exp.ArrayContainsAll: lambda self, e: self.binary(e, "@>"),
    501            exp.ArrayOverlaps: lambda self, e: self.binary(e, "&&"),
    502            exp.ArrayFilter: filter_array_using_unnest,
    503            exp.ArraySize: lambda self, e: self.func("ARRAY_LENGTH", e.this, e.expression or "1"),
    504            exp.BitwiseXor: lambda self, e: self.binary(e, "#"),
    505            exp.ColumnDef: transforms.preprocess([_auto_increment_to_serial, _serial_to_generated]),
    506            exp.CurrentDate: no_paren_current_date_sql,
    507            exp.CurrentTimestamp: lambda *_: "CURRENT_TIMESTAMP",
    508            exp.CurrentUser: lambda *_: "CURRENT_USER",
    509            exp.DateAdd: _date_add_sql("+"),
    510            exp.DateDiff: _date_diff_sql,
    511            exp.DateStrToDate: datestrtodate_sql,
    512            exp.DateSub: _date_add_sql("-"),
    513            exp.Explode: rename_func("UNNEST"),
    514            exp.GroupConcat: _string_agg_sql,
    515            exp.IntDiv: rename_func("DIV"),
    516            exp.JSONExtract: _json_extract_sql("JSON_EXTRACT_PATH", "->"),
    517            exp.JSONExtractScalar: _json_extract_sql("JSON_EXTRACT_PATH_TEXT", "->>"),
    518            exp.JSONBExtract: lambda self, e: self.binary(e, "#>"),
    519            exp.JSONBExtractScalar: lambda self, e: self.binary(e, "#>>"),
    520            exp.JSONBContains: lambda self, e: self.binary(e, "?"),
    521            exp.ParseJSON: lambda self, e: self.sql(exp.cast(e.this, exp.DataType.Type.JSON)),
    522            exp.JSONPathKey: json_path_key_only_name,
    523            exp.JSONPathRoot: lambda *_: "",
    524            exp.JSONPathSubscript: lambda self, e: self.json_path_part(e.this),
    525            exp.LastDay: no_last_day_sql,
    526            exp.LogicalOr: rename_func("BOOL_OR"),
    527            exp.LogicalAnd: rename_func("BOOL_AND"),
    528            exp.Max: max_or_greatest,
    529            exp.MapFromEntries: no_map_from_entries_sql,
    530            exp.Min: min_or_least,
    531            exp.Merge: merge_without_target_sql,
    532            exp.PartitionedByProperty: lambda self, e: f"PARTITION BY {self.sql(e, 'this')}",
    533            exp.PercentileCont: transforms.preprocess(
    534                [transforms.add_within_group_for_percentiles]
    535            ),
    536            exp.PercentileDisc: transforms.preprocess(
    537                [transforms.add_within_group_for_percentiles]
    538            ),
    539            exp.Pivot: no_pivot_sql,
    540            exp.Pow: lambda self, e: self.binary(e, "^"),
    541            exp.Rand: rename_func("RANDOM"),
    542            exp.RegexpLike: lambda self, e: self.binary(e, "~"),
    543            exp.RegexpILike: lambda self, e: self.binary(e, "~*"),
    544            exp.Select: transforms.preprocess(
    545                [
    546                    transforms.eliminate_semi_and_anti_joins,
    547                    transforms.eliminate_qualify,
    548                ]
    549            ),
    550            exp.SHA2: sha256_sql,
    551            exp.StrPosition: str_position_sql,
    552            exp.StrToDate: lambda self, e: self.func("TO_DATE", e.this, self.format_time(e)),
    553            exp.StrToTime: lambda self, e: self.func("TO_TIMESTAMP", e.this, self.format_time(e)),
    554            exp.StructExtract: struct_extract_sql,
    555            exp.Substring: _substring_sql,
    556            exp.TimeFromParts: rename_func("MAKE_TIME"),
    557            exp.TimestampFromParts: rename_func("MAKE_TIMESTAMP"),
    558            exp.TimestampTrunc: timestamptrunc_sql(zone=True),
    559            exp.TimeStrToTime: timestrtotime_sql,
    560            exp.TimeToStr: lambda self, e: self.func("TO_CHAR", e.this, self.format_time(e)),
    561            exp.ToChar: lambda self, e: self.function_fallback_sql(e),
    562            exp.Trim: trim_sql,
    563            exp.TryCast: no_trycast_sql,
    564            exp.TsOrDsAdd: _date_add_sql("+"),
    565            exp.TsOrDsDiff: _date_diff_sql,
    566            exp.UnixToTime: lambda self, e: self.func("TO_TIMESTAMP", e.this),
    567            exp.TimeToUnix: lambda self, e: self.func(
    568                "DATE_PART", exp.Literal.string("epoch"), e.this
    569            ),
    570            exp.VariancePop: rename_func("VAR_POP"),
    571            exp.Variance: rename_func("VAR_SAMP"),
    572            exp.Xor: bool_xor_sql,
    573            exp.UnixToTime: _unix_to_time_sql,
    574        }
    575        TRANSFORMS.pop(exp.CommentColumnConstraint)
    576
    577        PROPERTIES_LOCATION = {
    578            **generator.Generator.PROPERTIES_LOCATION,
    579            exp.PartitionedByProperty: exp.Properties.Location.POST_SCHEMA,
    580            exp.TransientProperty: exp.Properties.Location.UNSUPPORTED,
    581            exp.VolatileProperty: exp.Properties.Location.UNSUPPORTED,
    582        }
    583
    584        def schemacommentproperty_sql(self, expression: exp.SchemaCommentProperty) -> str:
    585            self.unsupported("Table comments are not supported in the CREATE statement")
    586            return ""
    587
    588        def commentcolumnconstraint_sql(self, expression: exp.CommentColumnConstraint) -> str:
    589            self.unsupported("Column comments are not supported in the CREATE statement")
    590            return ""
    591
    592        def unnest_sql(self, expression: exp.Unnest) -> str:
    593            if len(expression.expressions) == 1:
    594                from sqlglot.optimizer.annotate_types import annotate_types
    595
    596                this = annotate_types(expression.expressions[0])
    597                if this.is_type("array<json>"):
    598                    while isinstance(this, exp.Cast):
    599                        this = this.this
    600
    601                    arg = self.sql(exp.cast(this, exp.DataType.Type.JSON))
    602                    alias = self.sql(expression, "alias")
    603                    alias = f" AS {alias}" if alias else ""
    604
    605                    if expression.args.get("offset"):
    606                        self.unsupported("Unsupported JSON_ARRAY_ELEMENTS with offset")
    607
    608                    return f"JSON_ARRAY_ELEMENTS({arg}){alias}"
    609
    610            return super().unnest_sql(expression)
    611
    612        def bracket_sql(self, expression: exp.Bracket) -> str:
    613            """Forms like ARRAY[1, 2, 3][3] aren't allowed; we need to wrap the ARRAY."""
    614            if isinstance(expression.this, exp.Array):
    615                expression.set("this", exp.paren(expression.this, copy=False))
    616
    617            return super().bracket_sql(expression)
    618
    619        def matchagainst_sql(self, expression: exp.MatchAgainst) -> str:
    620            this = self.sql(expression, "this")
    621            expressions = [f"{self.sql(e)} @@ {this}" for e in expression.expressions]
    622            sql = " OR ".join(expressions)
    623            return f"({sql})" if len(expressions) > 1 else sql
    624
    625        def alterset_sql(self, expression: exp.AlterSet) -> str:
    626            exprs = self.expressions(expression, flat=True)
    627            exprs = f"({exprs})" if exprs else ""
    628
    629            access_method = self.sql(expression, "access_method")
    630            access_method = f"ACCESS METHOD {access_method}" if access_method else ""
    631            tablespace = self.sql(expression, "tablespace")
    632            tablespace = f"TABLESPACE {tablespace}" if tablespace else ""
    633            option = self.sql(expression, "option")
    634
    635            return f"SET {exprs}{access_method}{tablespace}{option}"
    636
    637        def datatype_sql(self, expression: exp.DataType) -> str:
    638            if expression.is_type(exp.DataType.Type.ARRAY):
    639                if expression.expressions:
    640                    values = self.expressions(expression, key="values", flat=True)
    641                    return f"{self.expressions(expression, flat=True)}[{values}]"
    642                return "ARRAY"
    643            return super().datatype_sql(expression)
    644
    645        def cast_sql(self, expression: exp.Cast, safe_prefix: t.Optional[str] = None) -> str:
    646            this = expression.this
    647
    648            # Postgres casts DIV() to decimal for transpilation but when roundtripping it's superfluous
    649            if isinstance(this, exp.IntDiv) and expression.to == exp.DataType.build("decimal"):
    650                return self.sql(this)
    651
    652            return super().cast_sql(expression, safe_prefix=safe_prefix)

INDEX_OFFSET = 1

The base index offset for arrays.

TYPED_DIVISION = True

Whether the behavior of `a / b` depends on the types of `a` and `b`. False means `a / b` is always float division. True means `a / b` is integer division if both `a` and `b` are integers.

CONCAT_COALESCE = True

A `NULL` arg in `CONCAT` yields `NULL` by default, but in some dialects it yields an empty string.

NULL_ORDERING = 'nulls_are_large'

Default `NULL` ordering method to use if not explicitly set. Possible values: `"nulls_are_small"`, `"nulls_are_large"`, `"nulls_are_last"`

TIME_FORMAT = "'YYYY-MM-DD HH24:MI:SS'"
TABLESAMPLE_SIZE_IS_PERCENT = True

Whether a size in the table sample clause represents percentage.

TIME_MAPPING: Dict[str, str] = {'AM': '%p', 'PM': '%p', 'D': '%u', 'DD': '%d', 'DDD': '%j', 'FMDD': '%-d', 'FMDDD': '%-j', 'FMHH12': '%-I', 'FMHH24': '%-H', 'FMMI': '%-M', 'FMMM': '%-m', 'FMSS': '%-S', 'HH12': '%I', 'HH24': '%H', 'MI': '%M', 'MM': '%m', 'OF': '%z', 'SS': '%S', 'TMDay': '%A', 'TMDy': '%a', 'TMMon': '%b', 'TMMonth': '%B', 'TZ': '%Z', 'US': '%f', 'WW': '%U', 'YY': '%y', 'YYYY': '%Y'}

Associates this dialect's time formats with their equivalent Python `strftime` formats.

SUPPORTS_COLUMN_JOIN_MARKS = False

Whether the old-style outer join (+) syntax is supported.

tokenizer_class = <class 'Postgres.Tokenizer'>
jsonpath_tokenizer_class = <class 'sqlglot.tokens.JSONPathTokenizer'>
parser_class = <class 'Postgres.Parser'>
generator_class = <class 'Postgres.Generator'>
TIME_TRIE: Dict = {'A': {'M': {0: True}}, 'P': {'M': {0: True}}, 'D': {0: True, 'D': {0: True, 'D': {0: True}}}, 'F': {'M': {'D': {'D': {0: True, 'D': {0: True}}}, 'H': {'H': {'1': {'2': {0: True}}, '2': {'4': {0: True}}}}, 'M': {'I': {0: True}, 'M': {0: True}}, 'S': {'S': {0: True}}}}, 'H': {'H': {'1': {'2': {0: True}}, '2': {'4': {0: True}}}}, 'M': {'I': {0: True}, 'M': {0: True}}, 'O': {'F': {0: True}}, 'S': {'S': {0: True}}, 'T': {'M': {'D': {'a': {'y': {0: True}}, 'y': {0: True}}, 'M': {'o': {'n': {0: True, 't': {'h': {0: True}}}}}}, 'Z': {0: True}}, 'U': {'S': {0: True}}, 'W': {'W': {0: True}}, 'Y': {'Y': {0: True, 'Y': {'Y': {0: True}}}}}
FORMAT_TRIE: Dict = {'A': {'M': {0: True}}, 'P': {'M': {0: True}}, 'D': {0: True, 'D': {0: True, 'D': {0: True}}}, 'F': {'M': {'D': {'D': {0: True, 'D': {0: True}}}, 'H': {'H': {'1': {'2': {0: True}}, '2': {'4': {0: True}}}}, 'M': {'I': {0: True}, 'M': {0: True}}, 'S': {'S': {0: True}}}}, 'H': {'H': {'1': {'2': {0: True}}, '2': {'4': {0: True}}}}, 'M': {'I': {0: True}, 'M': {0: True}}, 'O': {'F': {0: True}}, 'S': {'S': {0: True}}, 'T': {'M': {'D': {'a': {'y': {0: True}}, 'y': {0: True}}, 'M': {'o': {'n': {0: True, 't': {'h': {0: True}}}}}}, 'Z': {0: True}}, 'U': {'S': {0: True}}, 'W': {'W': {0: True}}, 'Y': {'Y': {0: True, 'Y': {'Y': {0: True}}}}}
INVERSE_TIME_MAPPING: Dict[str, str] = {'%p': 'PM', '%u': 'D', '%d': 'DD', '%j': 'DDD', '%-d': 'FMDD', '%-j': 'FMDDD', '%-I': 'FMHH12', '%-H': 'FMHH24', '%-M': 'FMMI', '%-m': 'FMMM', '%-S': 'FMSS', '%I': 'HH12', '%H': 'HH24', '%M': 'MI', '%m': 'MM', '%z': 'OF', '%S': 'SS', '%A': 'TMDay', '%a': 'TMDy', '%b': 'TMMon', '%B': 'TMMonth', '%Z': 'TZ', '%f': 'US', '%U': 'WW', '%y': 'YY', '%Y': 'YYYY'}
INVERSE_TIME_TRIE: Dict = {'%': {'p': {0: True}, 'u': {0: True}, 'd': {0: True}, 'j': {0: True}, '-': {'d': {0: True}, 'j': {0: True}, 'I': {0: True}, 'H': {0: True}, 'M': {0: True}, 'm': {0: True}, 'S': {0: True}}, 'I': {0: True}, 'H': {0: True}, 'M': {0: True}, 'm': {0: True}, 'z': {0: True}, 'S': {0: True}, 'A': {0: True}, 'a': {0: True}, 'b': {0: True}, 'B': {0: True}, 'Z': {0: True}, 'f': {0: True}, 'U': {0: True}, 'y': {0: True}, 'Y': {0: True}}}
INVERSE_FORMAT_MAPPING: Dict[str, str] = {}
INVERSE_FORMAT_TRIE: Dict = {}
ESCAPED_SEQUENCES: Dict[str, str] = {}
QUOTE_START = "'"
QUOTE_END = "'"
IDENTIFIER_START = '"'
IDENTIFIER_END = '"'
BIT_START: Optional[str] = "b'"
BIT_END: Optional[str] = "'"
HEX_START: Optional[str] = "x'"
HEX_END: Optional[str] = "'"
BYTE_START: Optional[str] = "e'"
BYTE_END: Optional[str] = "'"
UNICODE_START: Optional[str] = None
UNICODE_END: Optional[str] = None

##### Inherited Members

sqlglot.dialects.dialect.Dialect
  Dialect
  WEEK_OFFSET
  UNNEST_COLUMN_ONLY
  ALIAS_POST_TABLESAMPLE
  NORMALIZATION_STRATEGY
  IDENTIFIERS_CAN_START_WITH_DIGIT
  DPIPE_IS_STRING_CONCAT
  STRICT_STRING_CONCAT
  SUPPORTS_USER_DEFINED_TYPES
  SUPPORTS_SEMI_ANTI_JOIN
  COPY_PARAMS_ARE_CSV
  NORMALIZE_FUNCTIONS
  LOG_BASE_FIRST
  SAFE_DIVISION
  HEX_LOWERCASE
  DATE_FORMAT
  DATEINT_FORMAT
  FORMAT_MAPPING
  UNESCAPED_SEQUENCES
  PSEUDOCOLUMNS
  PREFER_CTE_ALIAS_COLUMN
  FORCE_EARLY_ALIAS_REF_EXPANSION
  EXPAND_ALIAS_REFS_EARLY_ONLY_IN_GROUP_BY
  SUPPORTS_ORDER_BY_ALL
  DATE_PART_MAPPING
  TYPE_TO_EXPRESSIONS
  ANNOTATORS
  get_or_raise
  format_time
  settings
  normalize_identifier
  case_sensitive
  can_identify
  quote_identifier
  to_json_path
  parse
  parse_into
  generate
  transpile
  tokenize
  tokenizer
  jsonpath_tokenizer
  parser
  generator

class Postgres.Tokenizer(sqlglot.tokens.Tokenizer): View Source

    282    class Tokenizer(tokens.Tokenizer):
    283        BIT_STRINGS = [("b'", "'"), ("B'", "'")]
    284        HEX_STRINGS = [("x'", "'"), ("X'", "'")]
    285        BYTE_STRINGS = [("e'", "'"), ("E'", "'")]
    286        HEREDOC_STRINGS = ["$"]
    287
    288        HEREDOC_TAG_IS_IDENTIFIER = True
    289        HEREDOC_STRING_ALTERNATIVE = TokenType.PARAMETER
    290
    291        KEYWORDS = {
    292            **tokens.Tokenizer.KEYWORDS,
    293            "~~": TokenType.LIKE,
    294            "~~*": TokenType.ILIKE,
    295            "~*": TokenType.IRLIKE,
    296            "~": TokenType.RLIKE,
    297            "@@": TokenType.DAT,
    298            "@>": TokenType.AT_GT,
    299            "<@": TokenType.LT_AT,
    300            "|/": TokenType.PIPE_SLASH,
    301            "||/": TokenType.DPIPE_SLASH,
    302            "BEGIN": TokenType.COMMAND,
    303            "BEGIN TRANSACTION": TokenType.BEGIN,
    304            "BIGSERIAL": TokenType.BIGSERIAL,
    305            "CHARACTER VARYING": TokenType.VARCHAR,
    306            "CONSTRAINT TRIGGER": TokenType.COMMAND,
    307            "CSTRING": TokenType.PSEUDO_TYPE,
    308            "DECLARE": TokenType.COMMAND,
    309            "DO": TokenType.COMMAND,
    310            "EXEC": TokenType.COMMAND,
    311            "HSTORE": TokenType.HSTORE,
    312            "INT8": TokenType.BIGINT,
    313            "MONEY": TokenType.MONEY,
    314            "NAME": TokenType.NAME,
    315            "OID": TokenType.OBJECT_IDENTIFIER,
    316            "ONLY": TokenType.ONLY,
    317            "OPERATOR": TokenType.OPERATOR,
    318            "REFRESH": TokenType.COMMAND,
    319            "REINDEX": TokenType.COMMAND,
    320            "RESET": TokenType.COMMAND,
    321            "REVOKE": TokenType.COMMAND,
    322            "SERIAL": TokenType.SERIAL,
    323            "SMALLSERIAL": TokenType.SMALLSERIAL,
    324            "TEMP": TokenType.TEMPORARY,
    325            "REGCLASS": TokenType.OBJECT_IDENTIFIER,
    326            "REGCOLLATION": TokenType.OBJECT_IDENTIFIER,
    327            "REGCONFIG": TokenType.OBJECT_IDENTIFIER,
    328            "REGDICTIONARY": TokenType.OBJECT_IDENTIFIER,
    329            "REGNAMESPACE": TokenType.OBJECT_IDENTIFIER,
    330            "REGOPER": TokenType.OBJECT_IDENTIFIER,
    331            "REGOPERATOR": TokenType.OBJECT_IDENTIFIER,
    332            "REGPROC": TokenType.OBJECT_IDENTIFIER,
    333            "REGPROCEDURE": TokenType.OBJECT_IDENTIFIER,
    334            "REGROLE": TokenType.OBJECT_IDENTIFIER,
    335            "REGTYPE": TokenType.OBJECT_IDENTIFIER,
    336            "FLOAT": TokenType.DOUBLE,
    337        }
    338        KEYWORDS.pop("/*+")
    339        KEYWORDS.pop("DIV")
    340
    341        SINGLE_TOKENS = {
    342            **tokens.Tokenizer.SINGLE_TOKENS,
    343            "$": TokenType.HEREDOC_STRING,
    344        }
    345
    346        VAR_SINGLE_TOKENS = {"$"}

BIT_STRINGS = [("b'", "'"), ("B'", "'")]
HEX_STRINGS = [("x'", "'"), ("X'", "'")]
BYTE_STRINGS = [("e'", "'"), ("E'", "'")]
HEREDOC_STRINGS = ['$']
HEREDOC_TAG_IS_IDENTIFIER = True
HEREDOC_STRING_ALTERNATIVE = <TokenType.PARAMETER: 'PARAMETER'>
KEYWORDS = {'{%': <TokenType.BLOCK_START: 'BLOCK_START'>, '{%+': <TokenType.BLOCK_START: 'BLOCK_START'>, '{%-': <TokenType.BLOCK_START: 'BLOCK_START'>, '%}': <TokenType.BLOCK_END: 'BLOCK_END'>, '+%}': <TokenType.BLOCK_END: 'BLOCK_END'>, '-%}': <TokenType.BLOCK_END: 'BLOCK_END'>, '{{+': <TokenType.BLOCK_START: 'BLOCK_START'>, '{{-': <TokenType.BLOCK_START: 'BLOCK_START'>, '+}}': <TokenType.BLOCK_END: 'BLOCK_END'>, '-}}': <TokenType.BLOCK_END: 'BLOCK_END'>, '==': <TokenType.EQ: 'EQ'>, '::': <TokenType.DCOLON: 'DCOLON'>, '||': <TokenType.DPIPE: 'DPIPE'>, '>=': <TokenType.GTE: 'GTE'>, '<=': <TokenType.LTE: 'LTE'>, '<>': <TokenType.NEQ: 'NEQ'>, '!=': <TokenType.NEQ: 'NEQ'>, ':=': <TokenType.COLON_EQ: 'COLON_EQ'>, '<=>': <TokenType.NULLSAFE_EQ: 'NULLSAFE_EQ'>, '->': <TokenType.ARROW: 'ARROW'>, '->>': <TokenType.DARROW: 'DARROW'>, '=>': <TokenType.FARROW: 'FARROW'>, '#>': <TokenType.HASH_ARROW: 'HASH_ARROW'>, '#>>': <TokenType.DHASH_ARROW: 'DHASH_ARROW'>, '<->': <TokenType.LR_ARROW: 'LR_ARROW'>, '&&': <TokenType.DAMP: 'DAMP'>, '??': <TokenType.DQMARK: 'DQMARK'>, 'ALL': <TokenType.ALL: 'ALL'>, 'ALWAYS': <TokenType.ALWAYS: 'ALWAYS'>, 'AND': <TokenType.AND: 'AND'>, 'ANTI': <TokenType.ANTI: 'ANTI'>, 'ANY': <TokenType.ANY: 'ANY'>, 'ASC': <TokenType.ASC: 'ASC'>, 'AS': <TokenType.ALIAS: 'ALIAS'>, 'ASOF': <TokenType.ASOF: 'ASOF'>, 'AUTOINCREMENT': <TokenType.AUTO_INCREMENT: 'AUTO_INCREMENT'>, 'AUTO_INCREMENT': <TokenType.AUTO_INCREMENT: 'AUTO_INCREMENT'>, 'BEGIN': <TokenType.COMMAND: 'COMMAND'>, 'BETWEEN': <TokenType.BETWEEN: 'BETWEEN'>, 'CACHE': <TokenType.CACHE: 'CACHE'>, 'UNCACHE': <TokenType.UNCACHE: 'UNCACHE'>, 'CASE': <TokenType.CASE: 'CASE'>, 'CHARACTER SET': <TokenType.CHARACTER_SET: 'CHARACTER_SET'>, 'CLUSTER BY': <TokenType.CLUSTER_BY: 'CLUSTER_BY'>, 'COLLATE': <TokenType.COLLATE: 'COLLATE'>, 'COLUMN': <TokenType.COLUMN: 'COLUMN'>, 'COMMIT': <TokenType.COMMIT: 'COMMIT'>, 'CONNECT BY': <TokenType.CONNECT_BY: 'CONNECT_BY'>, 'CONSTRAINT': <TokenType.CONSTRAINT: 'CONSTRAINT'>, 'COPY': <TokenType.COPY: 'COPY'>, 'CREATE': <TokenType.CREATE: 'CREATE'>, 'CROSS': <TokenType.CROSS: 'CROSS'>, 'CUBE': <TokenType.CUBE: 'CUBE'>, 'CURRENT_DATE': <TokenType.CURRENT_DATE: 'CURRENT_DATE'>, 'CURRENT_TIME': <TokenType.CURRENT_TIME: 'CURRENT_TIME'>, 'CURRENT_TIMESTAMP': <TokenType.CURRENT_TIMESTAMP: 'CURRENT_TIMESTAMP'>, 'CURRENT_USER': <TokenType.CURRENT_USER: 'CURRENT_USER'>, 'DATABASE': <TokenType.DATABASE: 'DATABASE'>, 'DEFAULT': <TokenType.DEFAULT: 'DEFAULT'>, 'DELETE': <TokenType.DELETE: 'DELETE'>, 'DESC': <TokenType.DESC: 'DESC'>, 'DESCRIBE': <TokenType.DESCRIBE: 'DESCRIBE'>, 'DISTINCT': <TokenType.DISTINCT: 'DISTINCT'>, 'DISTRIBUTE BY': <TokenType.DISTRIBUTE_BY: 'DISTRIBUTE_BY'>, 'DROP': <TokenType.DROP: 'DROP'>, 'ELSE': <TokenType.ELSE: 'ELSE'>, 'END': <TokenType.END: 'END'>, 'ENUM': <TokenType.ENUM: 'ENUM'>, 'ESCAPE': <TokenType.ESCAPE: 'ESCAPE'>, 'EXCEPT': <TokenType.EXCEPT: 'EXCEPT'>, 'EXECUTE': <TokenType.EXECUTE: 'EXECUTE'>, 'EXISTS': <TokenType.EXISTS: 'EXISTS'>, 'FALSE': <TokenType.FALSE: 'FALSE'>, 'FETCH': <TokenType.FETCH: 'FETCH'>, 'FILTER': <TokenType.FILTER: 'FILTER'>, 'FIRST': <TokenType.FIRST: 'FIRST'>, 'FULL': <TokenType.FULL: 'FULL'>, 'FUNCTION': <TokenType.FUNCTION: 'FUNCTION'>, 'FOR': <TokenType.FOR: 'FOR'>, 'FOREIGN KEY': <TokenType.FOREIGN_KEY: 'FOREIGN_KEY'>, 'FORMAT': <TokenType.FORMAT: 'FORMAT'>, 'FROM': <TokenType.FROM: 'FROM'>, 'GEOGRAPHY': <TokenType.GEOGRAPHY: 'GEOGRAPHY'>, 'GEOMETRY': <TokenType.GEOMETRY: 'GEOMETRY'>, 'GLOB': <TokenType.GLOB: 'GLOB'>, 'GROUP BY': <TokenType.GROUP_BY: 'GROUP_BY'>, 'GROUPING SETS': <TokenType.GROUPING_SETS: 'GROUPING_SETS'>, 'HAVING': <TokenType.HAVING: 'HAVING'>, 'ILIKE': <TokenType.ILIKE: 'ILIKE'>, 'IN': <TokenType.IN: 'IN'>, 'INDEX': <TokenType.INDEX: 'INDEX'>, 'INET': <TokenType.INET: 'INET'>, 'INNER': <TokenType.INNER: 'INNER'>, 'INSERT': <TokenType.INSERT: 'INSERT'>, 'INTERVAL': <TokenType.INTERVAL: 'INTERVAL'>, 'INTERSECT': <TokenType.INTERSECT: 'INTERSECT'>, 'INTO': <TokenType.INTO: 'INTO'>, 'IS': <TokenType.IS: 'IS'>, 'ISNULL': <TokenType.ISNULL: 'ISNULL'>, 'JOIN': <TokenType.JOIN: 'JOIN'>, 'KEEP': <TokenType.KEEP: 'KEEP'>, 'KILL': <TokenType.KILL: 'KILL'>, 'LATERAL': <TokenType.LATERAL: 'LATERAL'>, 'LEFT': <TokenType.LEFT: 'LEFT'>, 'LIKE': <TokenType.LIKE: 'LIKE'>, 'LIMIT': <TokenType.LIMIT: 'LIMIT'>, 'LOAD': <TokenType.LOAD: 'LOAD'>, 'LOCK': <TokenType.LOCK: 'LOCK'>, 'MERGE': <TokenType.MERGE: 'MERGE'>, 'NATURAL': <TokenType.NATURAL: 'NATURAL'>, 'NEXT': <TokenType.NEXT: 'NEXT'>, 'NOT': <TokenType.NOT: 'NOT'>, 'NOTNULL': <TokenType.NOTNULL: 'NOTNULL'>, 'NULL': <TokenType.NULL: 'NULL'>, 'OBJECT': <TokenType.OBJECT: 'OBJECT'>, 'OFFSET': <TokenType.OFFSET: 'OFFSET'>, 'ON': <TokenType.ON: 'ON'>, 'OR': <TokenType.OR: 'OR'>, 'XOR': <TokenType.XOR: 'XOR'>, 'ORDER BY': <TokenType.ORDER_BY: 'ORDER_BY'>, 'ORDINALITY': <TokenType.ORDINALITY: 'ORDINALITY'>, 'OUTER': <TokenType.OUTER: 'OUTER'>, 'OVER': <TokenType.OVER: 'OVER'>, 'OVERLAPS': <TokenType.OVERLAPS: 'OVERLAPS'>, 'OVERWRITE': <TokenType.OVERWRITE: 'OVERWRITE'>, 'PARTITION': <TokenType.PARTITION: 'PARTITION'>, 'PARTITION BY': <TokenType.PARTITION_BY: 'PARTITION_BY'>, 'PARTITIONED BY': <TokenType.PARTITION_BY: 'PARTITION_BY'>, 'PARTITIONED_BY': <TokenType.PARTITION_BY: 'PARTITION_BY'>, 'PERCENT': <TokenType.PERCENT: 'PERCENT'>, 'PIVOT': <TokenType.PIVOT: 'PIVOT'>, 'PRAGMA': <TokenType.PRAGMA: 'PRAGMA'>, 'PRIMARY KEY': <TokenType.PRIMARY_KEY: 'PRIMARY_KEY'>, 'PROCEDURE': <TokenType.PROCEDURE: 'PROCEDURE'>, 'QUALIFY': <TokenType.QUALIFY: 'QUALIFY'>, 'RANGE': <TokenType.RANGE: 'RANGE'>, 'RECURSIVE': <TokenType.RECURSIVE: 'RECURSIVE'>, 'REGEXP': <TokenType.RLIKE: 'RLIKE'>, 'REPLACE': <TokenType.REPLACE: 'REPLACE'>, 'RETURNING': <TokenType.RETURNING: 'RETURNING'>, 'REFERENCES': <TokenType.REFERENCES: 'REFERENCES'>, 'RIGHT': <TokenType.RIGHT: 'RIGHT'>, 'RLIKE': <TokenType.RLIKE: 'RLIKE'>, 'ROLLBACK': <TokenType.ROLLBACK: 'ROLLBACK'>, 'ROLLUP': <TokenType.ROLLUP: 'ROLLUP'>, 'ROW': <TokenType.ROW: 'ROW'>, 'ROWS': <TokenType.ROWS: 'ROWS'>, 'SCHEMA': <TokenType.SCHEMA: 'SCHEMA'>, 'SELECT': <TokenType.SELECT: 'SELECT'>, 'SEMI': <TokenType.SEMI: 'SEMI'>, 'SET': <TokenType.SET: 'SET'>, 'SETTINGS': <TokenType.SETTINGS: 'SETTINGS'>, 'SHOW': <TokenType.SHOW: 'SHOW'>, 'SIMILAR TO': <TokenType.SIMILAR_TO: 'SIMILAR_TO'>, 'SOME': <TokenType.SOME: 'SOME'>, 'SORT BY': <TokenType.SORT_BY: 'SORT_BY'>, 'START WITH': <TokenType.START_WITH: 'START_WITH'>, 'STRAIGHT_JOIN': <TokenType.STRAIGHT_JOIN: 'STRAIGHT_JOIN'>, 'TABLE': <TokenType.TABLE: 'TABLE'>, 'TABLESAMPLE': <TokenType.TABLE_SAMPLE: 'TABLE_SAMPLE'>, 'TEMP': <TokenType.TEMPORARY: 'TEMPORARY'>, 'TEMPORARY': <TokenType.TEMPORARY: 'TEMPORARY'>, 'THEN': <TokenType.THEN: 'THEN'>, 'TRUE': <TokenType.TRUE: 'TRUE'>, 'TRUNCATE': <TokenType.TRUNCATE: 'TRUNCATE'>, 'UNION': <TokenType.UNION: 'UNION'>, 'UNKNOWN': <TokenType.UNKNOWN: 'UNKNOWN'>, 'UNNEST': <TokenType.UNNEST: 'UNNEST'>, 'UNPIVOT': <TokenType.UNPIVOT: 'UNPIVOT'>, 'UPDATE': <TokenType.UPDATE: 'UPDATE'>, 'USE': <TokenType.USE: 'USE'>, 'USING': <TokenType.USING: 'USING'>, 'UUID': <TokenType.UUID: 'UUID'>, 'VALUES': <TokenType.VALUES: 'VALUES'>, 'VIEW': <TokenType.VIEW: 'VIEW'>, 'VOLATILE': <TokenType.VOLATILE: 'VOLATILE'>, 'WHEN': <TokenType.WHEN: 'WHEN'>, 'WHERE': <TokenType.WHERE: 'WHERE'>, 'WINDOW': <TokenType.WINDOW: 'WINDOW'>, 'WITH': <TokenType.WITH: 'WITH'>, 'APPLY': <TokenType.APPLY: 'APPLY'>, 'ARRAY': <TokenType.ARRAY: 'ARRAY'>, 'BIT': <TokenType.BIT: 'BIT'>, 'BOOL': <TokenType.BOOLEAN: 'BOOLEAN'>, 'BOOLEAN': <TokenType.BOOLEAN: 'BOOLEAN'>, 'BYTE': <TokenType.TINYINT: 'TINYINT'>, 'MEDIUMINT': <TokenType.MEDIUMINT: 'MEDIUMINT'>, 'INT1': <TokenType.TINYINT: 'TINYINT'>, 'TINYINT': <TokenType.TINYINT: 'TINYINT'>, 'INT16': <TokenType.SMALLINT: 'SMALLINT'>, 'SHORT': <TokenType.SMALLINT: 'SMALLINT'>, 'SMALLINT': <TokenType.SMALLINT: 'SMALLINT'>, 'INT128': <TokenType.INT128: 'INT128'>, 'HUGEINT': <TokenType.INT128: 'INT128'>, 'INT2': <TokenType.SMALLINT: 'SMALLINT'>, 'INTEGER': <TokenType.INT: 'INT'>, 'INT': <TokenType.INT: 'INT'>, 'INT4': <TokenType.INT: 'INT'>, 'INT32': <TokenType.INT: 'INT'>, 'INT64': <TokenType.BIGINT: 'BIGINT'>, 'LONG': <TokenType.BIGINT: 'BIGINT'>, 'BIGINT': <TokenType.BIGINT: 'BIGINT'>, 'INT8': <TokenType.BIGINT: 'BIGINT'>, 'UINT': <TokenType.UINT: 'UINT'>, 'DEC': <TokenType.DECIMAL: 'DECIMAL'>, 'DECIMAL': <TokenType.DECIMAL: 'DECIMAL'>, 'BIGDECIMAL': <TokenType.BIGDECIMAL: 'BIGDECIMAL'>, 'BIGNUMERIC': <TokenType.BIGDECIMAL: 'BIGDECIMAL'>, 'LIST': <TokenType.LIST: 'LIST'>, 'MAP': <TokenType.MAP: 'MAP'>, 'NULLABLE': <TokenType.NULLABLE: 'NULLABLE'>, 'NUMBER': <TokenType.DECIMAL: 'DECIMAL'>, 'NUMERIC': <TokenType.DECIMAL: 'DECIMAL'>, 'FIXED': <TokenType.DECIMAL: 'DECIMAL'>, 'REAL': <TokenType.FLOAT: 'FLOAT'>, 'FLOAT': <TokenType.DOUBLE: 'DOUBLE'>, 'FLOAT4': <TokenType.FLOAT: 'FLOAT'>, 'FLOAT8': <TokenType.DOUBLE: 'DOUBLE'>, 'DOUBLE': <TokenType.DOUBLE: 'DOUBLE'>, 'DOUBLE PRECISION': <TokenType.DOUBLE: 'DOUBLE'>, 'JSON': <TokenType.JSON: 'JSON'>, 'JSONB': <TokenType.JSONB: 'JSONB'>, 'CHAR': <TokenType.CHAR: 'CHAR'>, 'CHARACTER': <TokenType.CHAR: 'CHAR'>, 'NCHAR': <TokenType.NCHAR: 'NCHAR'>, 'VARCHAR': <TokenType.VARCHAR: 'VARCHAR'>, 'VARCHAR2': <TokenType.VARCHAR: 'VARCHAR'>, 'NVARCHAR': <TokenType.NVARCHAR: 'NVARCHAR'>, 'NVARCHAR2': <TokenType.NVARCHAR: 'NVARCHAR'>, 'BPCHAR': <TokenType.BPCHAR: 'BPCHAR'>, 'STR': <TokenType.TEXT: 'TEXT'>, 'STRING': <TokenType.TEXT: 'TEXT'>, 'TEXT': <TokenType.TEXT: 'TEXT'>, 'LONGTEXT': <TokenType.LONGTEXT: 'LONGTEXT'>, 'MEDIUMTEXT': <TokenType.MEDIUMTEXT: 'MEDIUMTEXT'>, 'TINYTEXT': <TokenType.TINYTEXT: 'TINYTEXT'>, 'CLOB': <TokenType.TEXT: 'TEXT'>, 'LONGVARCHAR': <TokenType.TEXT: 'TEXT'>, 'BINARY': <TokenType.BINARY: 'BINARY'>, 'BLOB': <TokenType.VARBINARY: 'VARBINARY'>, 'LONGBLOB': <TokenType.LONGBLOB: 'LONGBLOB'>, 'MEDIUMBLOB': <TokenType.MEDIUMBLOB: 'MEDIUMBLOB'>, 'TINYBLOB': <TokenType.TINYBLOB: 'TINYBLOB'>, 'BYTEA': <TokenType.VARBINARY: 'VARBINARY'>, 'VARBINARY': <TokenType.VARBINARY: 'VARBINARY'>, 'TIME': <TokenType.TIME: 'TIME'>, 'TIMETZ': <TokenType.TIMETZ: 'TIMETZ'>, 'TIMESTAMP': <TokenType.TIMESTAMP: 'TIMESTAMP'>, 'TIMESTAMPTZ': <TokenType.TIMESTAMPTZ: 'TIMESTAMPTZ'>, 'TIMESTAMPLTZ': <TokenType.TIMESTAMPLTZ: 'TIMESTAMPLTZ'>, 'TIMESTAMP_LTZ': <TokenType.TIMESTAMPLTZ: 'TIMESTAMPLTZ'>, 'TIMESTAMPNTZ': <TokenType.TIMESTAMPNTZ: 'TIMESTAMPNTZ'>, 'TIMESTAMP_NTZ': <TokenType.TIMESTAMPNTZ: 'TIMESTAMPNTZ'>, 'DATE': <TokenType.DATE: 'DATE'>, 'DATETIME': <TokenType.DATETIME: 'DATETIME'>, 'INT4RANGE': <TokenType.INT4RANGE: 'INT4RANGE'>, 'INT4MULTIRANGE': <TokenType.INT4MULTIRANGE: 'INT4MULTIRANGE'>, 'INT8RANGE': <TokenType.INT8RANGE: 'INT8RANGE'>, 'INT8MULTIRANGE': <TokenType.INT8MULTIRANGE: 'INT8MULTIRANGE'>, 'NUMRANGE': <TokenType.NUMRANGE: 'NUMRANGE'>, 'NUMMULTIRANGE': <TokenType.NUMMULTIRANGE: 'NUMMULTIRANGE'>, 'TSRANGE': <TokenType.TSRANGE: 'TSRANGE'>, 'TSMULTIRANGE': <TokenType.TSMULTIRANGE: 'TSMULTIRANGE'>, 'TSTZRANGE': <TokenType.TSTZRANGE: 'TSTZRANGE'>, 'TSTZMULTIRANGE': <TokenType.TSTZMULTIRANGE: 'TSTZMULTIRANGE'>, 'DATERANGE': <TokenType.DATERANGE: 'DATERANGE'>, 'DATEMULTIRANGE': <TokenType.DATEMULTIRANGE: 'DATEMULTIRANGE'>, 'UNIQUE': <TokenType.UNIQUE: 'UNIQUE'>, 'VECTOR': <TokenType.VECTOR: 'VECTOR'>, 'STRUCT': <TokenType.STRUCT: 'STRUCT'>, 'SEQUENCE': <TokenType.SEQUENCE: 'SEQUENCE'>, 'VARIANT': <TokenType.VARIANT: 'VARIANT'>, 'ALTER': <TokenType.ALTER: 'ALTER'>, 'ANALYZE': <TokenType.COMMAND: 'COMMAND'>, 'CALL': <TokenType.COMMAND: 'COMMAND'>, 'COMMENT': <TokenType.COMMENT: 'COMMENT'>, 'EXPLAIN': <TokenType.COMMAND: 'COMMAND'>, 'GRANT': <TokenType.COMMAND: 'COMMAND'>, 'OPTIMIZE': <TokenType.COMMAND: 'COMMAND'>, 'PREPARE': <TokenType.COMMAND: 'COMMAND'>, 'VACUUM': <TokenType.COMMAND: 'COMMAND'>, 'USER-DEFINED': <TokenType.USERDEFINED: 'USERDEFINED'>, 'FOR VERSION': <TokenType.VERSION_SNAPSHOT: 'VERSION_SNAPSHOT'>, 'FOR TIMESTAMP': <TokenType.TIMESTAMP_SNAPSHOT: 'TIMESTAMP_SNAPSHOT'>, '~~': <TokenType.LIKE: 'LIKE'>, '~~*': <TokenType.ILIKE: 'ILIKE'>, '~*': <TokenType.IRLIKE: 'IRLIKE'>, '~': <TokenType.RLIKE: 'RLIKE'>, '@@': <TokenType.DAT: 'DAT'>, '@>': <TokenType.AT_GT: 'AT_GT'>, '<@': <TokenType.LT_AT: 'LT_AT'>, '|/': <TokenType.PIPE_SLASH: 'PIPE_SLASH'>, '||/': <TokenType.DPIPE_SLASH: 'DPIPE_SLASH'>, 'BEGIN TRANSACTION': <TokenType.BEGIN: 'BEGIN'>, 'BIGSERIAL': <TokenType.BIGSERIAL: 'BIGSERIAL'>, 'CHARACTER VARYING': <TokenType.VARCHAR: 'VARCHAR'>, 'CONSTRAINT TRIGGER': <TokenType.COMMAND: 'COMMAND'>, 'CSTRING': <TokenType.PSEUDO_TYPE: 'PSEUDO_TYPE'>, 'DECLARE': <TokenType.COMMAND: 'COMMAND'>, 'DO': <TokenType.COMMAND: 'COMMAND'>, 'EXEC': <TokenType.COMMAND: 'COMMAND'>, 'HSTORE': <TokenType.HSTORE: 'HSTORE'>, 'MONEY': <TokenType.MONEY: 'MONEY'>, 'NAME': <TokenType.NAME: 'NAME'>, 'OID': <TokenType.OBJECT_IDENTIFIER: 'OBJECT_IDENTIFIER'>, 'ONLY': <TokenType.ONLY: 'ONLY'>, 'OPERATOR': <TokenType.OPERATOR: 'OPERATOR'>, 'REFRESH': <TokenType.COMMAND: 'COMMAND'>, 'REINDEX': <TokenType.COMMAND: 'COMMAND'>, 'RESET': <TokenType.COMMAND: 'COMMAND'>, 'REVOKE': <TokenType.COMMAND: 'COMMAND'>, 'SERIAL': <TokenType.SERIAL: 'SERIAL'>, 'SMALLSERIAL': <TokenType.SMALLSERIAL: 'SMALLSERIAL'>, 'REGCLASS': <TokenType.OBJECT_IDENTIFIER: 'OBJECT_IDENTIFIER'>, 'REGCOLLATION': <TokenType.OBJECT_IDENTIFIER: 'OBJECT_IDENTIFIER'>, 'REGCONFIG': <TokenType.OBJECT_IDENTIFIER: 'OBJECT_IDENTIFIER'>, 'REGDICTIONARY': <TokenType.OBJECT_IDENTIFIER: 'OBJECT_IDENTIFIER'>, 'REGNAMESPACE': <TokenType.OBJECT_IDENTIFIER: 'OBJECT_IDENTIFIER'>, 'REGOPER': <TokenType.OBJECT_IDENTIFIER: 'OBJECT_IDENTIFIER'>, 'REGOPERATOR': <TokenType.OBJECT_IDENTIFIER: 'OBJECT_IDENTIFIER'>, 'REGPROC': <TokenType.OBJECT_IDENTIFIER: 'OBJECT_IDENTIFIER'>, 'REGPROCEDURE': <TokenType.OBJECT_IDENTIFIER: 'OBJECT_IDENTIFIER'>, 'REGROLE': <TokenType.OBJECT_IDENTIFIER: 'OBJECT_IDENTIFIER'>, 'REGTYPE': <TokenType.OBJECT_IDENTIFIER: 'OBJECT_IDENTIFIER'>}
SINGLE_TOKENS = {'(': <TokenType.L_PAREN: 'L_PAREN'>, ')': <TokenType.R_PAREN: 'R_PAREN'>, '[': <TokenType.L_BRACKET: 'L_BRACKET'>, ']': <TokenType.R_BRACKET: 'R_BRACKET'>, '{': <TokenType.L_BRACE: 'L_BRACE'>, '}': <TokenType.R_BRACE: 'R_BRACE'>, '&': <TokenType.AMP: 'AMP'>, '^': <TokenType.CARET: 'CARET'>, ':': <TokenType.COLON: 'COLON'>, ',': <TokenType.COMMA: 'COMMA'>, '.': <TokenType.DOT: 'DOT'>, '-': <TokenType.DASH: 'DASH'>, '=': <TokenType.EQ: 'EQ'>, '>': <TokenType.GT: 'GT'>, '<': <TokenType.LT: 'LT'>, '%': <TokenType.MOD: 'MOD'>, '!': <TokenType.NOT: 'NOT'>, '|': <TokenType.PIPE: 'PIPE'>, '+': <TokenType.PLUS: 'PLUS'>, ';': <TokenType.SEMICOLON: 'SEMICOLON'>, '/': <TokenType.SLASH: 'SLASH'>, '\\': <TokenType.BACKSLASH: 'BACKSLASH'>, '*': <TokenType.STAR: 'STAR'>, '~': <TokenType.TILDA: 'TILDA'>, '?': <TokenType.PLACEHOLDER: 'PLACEHOLDER'>, '@': <TokenType.PARAMETER: 'PARAMETER'>, '#': <TokenType.HASH: 'HASH'>, "'": <TokenType.UNKNOWN: 'UNKNOWN'>, '`': <TokenType.UNKNOWN: 'UNKNOWN'>, '"': <TokenType.UNKNOWN: 'UNKNOWN'>, '$': <TokenType.HEREDOC_STRING: 'HEREDOC_STRING'>}
VAR_SINGLE_TOKENS = {'$'}

##### Inherited Members

sqlglot.tokens.Tokenizer
  Tokenizer
  RAW_STRINGS
  UNICODE_STRINGS
  IDENTIFIERS
  IDENTIFIER_ESCAPES
  QUOTES
  STRING_ESCAPES
  STRING_ESCAPES_ALLOWED_IN_RAW_STRINGS
  WHITE_SPACE
  COMMANDS
  COMMAND_PREFIX_TOKENS
  NUMERIC_LITERALS
  COMMENTS
  dialect
  reset
  tokenize
  tokenize_rs
  size
  sql
  tokens

class Postgres.Parser(sqlglot.parser.Parser): View Source

    348    class Parser(parser.Parser):
    349        PROPERTY_PARSERS = {
    350            **parser.Parser.PROPERTY_PARSERS,
    351            "SET": lambda self: self.expression(exp.SetConfigProperty, this=self._parse_set()),
    352        }
    353        PROPERTY_PARSERS.pop("INPUT")
    354
    355        FUNCTIONS = {
    356            **parser.Parser.FUNCTIONS,
    357            "DATE_TRUNC": build_timestamp_trunc,
    358            "DIV": lambda args: exp.cast(
    359                binary_from_function(exp.IntDiv)(args), exp.DataType.Type.DECIMAL
    360            ),
    361            "GENERATE_SERIES": _build_generate_series,
    362            "JSON_EXTRACT_PATH": build_json_extract_path(exp.JSONExtract),
    363            "JSON_EXTRACT_PATH_TEXT": build_json_extract_path(exp.JSONExtractScalar),
    364            "MAKE_TIME": exp.TimeFromParts.from_arg_list,
    365            "MAKE_TIMESTAMP": exp.TimestampFromParts.from_arg_list,
    366            "NOW": exp.CurrentTimestamp.from_arg_list,
    367            "REGEXP_REPLACE": _build_regexp_replace,
    368            "TO_CHAR": build_formatted_time(exp.TimeToStr, "postgres"),
    369            "TO_DATE": build_formatted_time(exp.StrToDate, "postgres"),
    370            "TO_TIMESTAMP": _build_to_timestamp,
    371            "UNNEST": exp.Explode.from_arg_list,
    372            "SHA256": lambda args: exp.SHA2(this=seq_get(args, 0), length=exp.Literal.number(256)),
    373            "SHA384": lambda args: exp.SHA2(this=seq_get(args, 0), length=exp.Literal.number(384)),
    374            "SHA512": lambda args: exp.SHA2(this=seq_get(args, 0), length=exp.Literal.number(512)),
    375        }
    376
    377        FUNCTION_PARSERS = {
    378            **parser.Parser.FUNCTION_PARSERS,
    379            "DATE_PART": lambda self: self._parse_date_part(),
    380        }
    381
    382        BITWISE = {
    383            **parser.Parser.BITWISE,
    384            TokenType.HASH: exp.BitwiseXor,
    385        }
    386
    387        EXPONENT = {
    388            TokenType.CARET: exp.Pow,
    389        }
    390
    391        RANGE_PARSERS = {
    392            **parser.Parser.RANGE_PARSERS,
    393            TokenType.AT_GT: binary_range_parser(exp.ArrayContainsAll),
    394            TokenType.DAMP: binary_range_parser(exp.ArrayOverlaps),
    395            TokenType.DAT: lambda self, this: self.expression(
    396                exp.MatchAgainst, this=self._parse_bitwise(), expressions=[this]
    397            ),
    398            TokenType.LT_AT: binary_range_parser(exp.ArrayContainsAll, reverse_args=True),
    399            TokenType.OPERATOR: lambda self, this: self._parse_operator(this),
    400        }
    401
    402        STATEMENT_PARSERS = {
    403            **parser.Parser.STATEMENT_PARSERS,
    404            TokenType.END: lambda self: self._parse_commit_or_rollback(),
    405        }
    406
    407        JSON_ARROWS_REQUIRE_JSON_TYPE = True
    408
    409        COLUMN_OPERATORS = {
    410            **parser.Parser.COLUMN_OPERATORS,
    411            TokenType.ARROW: lambda self, this, path: build_json_extract_path(
    412                exp.JSONExtract, arrow_req_json_type=self.JSON_ARROWS_REQUIRE_JSON_TYPE
    413            )([this, path]),
    414            TokenType.DARROW: lambda self, this, path: build_json_extract_path(
    415                exp.JSONExtractScalar, arrow_req_json_type=self.JSON_ARROWS_REQUIRE_JSON_TYPE
    416            )([this, path]),
    417        }
    418
    419        def _parse_operator(self, this: t.Optional[exp.Expression]) -> t.Optional[exp.Expression]:
    420            while True:
    421                if not self._match(TokenType.L_PAREN):
    422                    break
    423
    424                op = ""
    425                while self._curr and not self._match(TokenType.R_PAREN):
    426                    op += self._curr.text
    427                    self._advance()
    428
    429                this = self.expression(
    430                    exp.Operator,
    431                    comments=self._prev_comments,
    432                    this=this,
    433                    operator=op,
    434                    expression=self._parse_bitwise(),
    435                )
    436
    437                if not self._match(TokenType.OPERATOR):
    438                    break
    439
    440            return this
    441
    442        def _parse_date_part(self) -> exp.Expression:
    443            part = self._parse_type()
    444            self._match(TokenType.COMMA)
    445            value = self._parse_bitwise()
    446
    447            if part and part.is_string:
    448                part = exp.var(part.name)
    449
    450            return self.expression(exp.Extract, this=part, expression=value)
    451
    452        def _parse_unique_key(self) -> t.Optional[exp.Expression]:
    453            return None

Parser consumes a list of tokens produced by the Tokenizer and produces a parsed syntax tree.

###### Arguments:

- error_level: The desired error level. Default: ErrorLevel.IMMEDIATE
- error_message_context: The amount of context to capture from a query string when displaying the error message (in number of characters). Default: 100
- max_errors: Maximum number of error messages to include in a raised ParseError. This is only relevant if error_level is ErrorLevel.RAISE. Default: 3

PROPERTY_PARSERS = {'ALLOWED_VALUES': <function Parser.<lambda>>, 'ALGORITHM': <function Parser.<lambda>>, 'AUTO': <function Parser.<lambda>>, 'AUTO_INCREMENT': <function Parser.<lambda>>, 'BACKUP': <function Parser.<lambda>>, 'BLOCKCOMPRESSION': <function Parser.<lambda>>, 'CHARSET': <function Parser.<lambda>>, 'CHARACTER SET': <function Parser.<lambda>>, 'CHECKSUM': <function Parser.<lambda>>, 'CLUSTER BY': <function Parser.<lambda>>, 'CLUSTERED': <function Parser.<lambda>>, 'COLLATE': <function Parser.<lambda>>, 'COMMENT': <function Parser.<lambda>>, 'CONTAINS': <function Parser.<lambda>>, 'COPY': <function Parser.<lambda>>, 'DATABLOCKSIZE': <function Parser.<lambda>>, 'DATA_DELETION': <function Parser.<lambda>>, 'DEFINER': <function Parser.<lambda>>, 'DETERMINISTIC': <function Parser.<lambda>>, 'DYNAMIC': <function Parser.<lambda>>, 'DISTKEY': <function Parser.<lambda>>, 'DISTSTYLE': <function Parser.<lambda>>, 'ENGINE': <function Parser.<lambda>>, 'EXECUTE': <function Parser.<lambda>>, 'EXTERNAL': <function Parser.<lambda>>, 'FALLBACK': <function Parser.<lambda>>, 'FORMAT': <function Parser.<lambda>>, 'FREESPACE': <function Parser.<lambda>>, 'GLOBAL': <function Parser.<lambda>>, 'HEAP': <function Parser.<lambda>>, 'ICEBERG': <function Parser.<lambda>>, 'IMMUTABLE': <function Parser.<lambda>>, 'INHERITS': <function Parser.<lambda>>, 'JOURNAL': <function Parser.<lambda>>, 'LANGUAGE': <function Parser.<lambda>>, 'LAYOUT': <function Parser.<lambda>>, 'LIFETIME': <function Parser.<lambda>>, 'LIKE': <function Parser.<lambda>>, 'LOCATION': <function Parser.<lambda>>, 'LOCK': <function Parser.<lambda>>, 'LOCKING': <function Parser.<lambda>>, 'LOG': <function Parser.<lambda>>, 'MATERIALIZED': <function Parser.<lambda>>, 'MERGEBLOCKRATIO': <function Parser.<lambda>>, 'MODIFIES': <function Parser.<lambda>>, 'MULTISET': <function Parser.<lambda>>, 'NO': <function Parser.<lambda>>, 'ON': <function Parser.<lambda>>, 'ORDER BY': <function Parser.<lambda>>, 'OUTPUT': <function Parser.<lambda>>, 'PARTITION': <function Parser.<lambda>>, 'PARTITION BY': <function Parser.<lambda>>, 'PARTITIONED BY': <function Parser.<lambda>>, 'PARTITIONED_BY': <function Parser.<lambda>>, 'PRIMARY KEY': <function Parser.<lambda>>, 'RANGE': <function Parser.<lambda>>, 'READS': <function Parser.<lambda>>, 'REMOTE': <function Parser.<lambda>>, 'RETURNS': <function Parser.<lambda>>, 'STRICT': <function Parser.<lambda>>, 'ROW': <function Parser.<lambda>>, 'ROW_FORMAT': <function Parser.<lambda>>, 'SAMPLE': <function Parser.<lambda>>, 'SECURE': <function Parser.<lambda>>, 'SET': <function Postgres.Parser.<lambda>>, 'SETTINGS': <function Parser.<lambda>>, 'SHARING': <function Parser.<lambda>>, 'SORTKEY': <function Parser.<lambda>>, 'SOURCE': <function Parser.<lambda>>, 'STABLE': <function Parser.<lambda>>, 'STORED': <function Parser.<lambda>>, 'SYSTEM_VERSIONING': <function Parser.<lambda>>, 'TBLPROPERTIES': <function Parser.<lambda>>, 'TEMP': <function Parser.<lambda>>, 'TEMPORARY': <function Parser.<lambda>>, 'TO': <function Parser.<lambda>>, 'TRANSIENT': <function Parser.<lambda>>, 'TRANSFORM': <function Parser.<lambda>>, 'TTL': <function Parser.<lambda>>, 'USING': <function Parser.<lambda>>, 'UNLOGGED': <function Parser.<lambda>>, 'VOLATILE': <function Parser.<lambda>>, 'WITH': <function Parser.<lambda>>}
FUNCTIONS = {'ABS': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Abs'>>, 'ADD_MONTHS': <bound method Func.from_arg_list of <class 'sqlglot.expressions.AddMonths'>>, 'ANONYMOUS_AGG_FUNC': <bound method Func.from_arg_list of <class 'sqlglot.expressions.AnonymousAggFunc'>>, 'ANY_VALUE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.AnyValue'>>, 'APPROX_DISTINCT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ApproxDistinct'>>, 'APPROX_COUNT_DISTINCT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ApproxDistinct'>>, 'APPROX_QUANTILE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ApproxQuantile'>>, 'APPROX_TOP_K': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ApproxTopK'>>, 'ARG_MAX': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArgMax'>>, 'ARGMAX': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArgMax'>>, 'MAX_BY': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArgMax'>>, 'ARG_MIN': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArgMin'>>, 'ARGMIN': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArgMin'>>, 'MIN_BY': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArgMin'>>, 'ARRAY': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Array'>>, 'ARRAY_AGG': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayAgg'>>, 'ARRAY_ALL': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayAll'>>, 'ARRAY_ANY': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayAny'>>, 'ARRAY_CONCAT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayConcat'>>, 'ARRAY_CAT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayConcat'>>, 'ARRAY_CONSTRUCT_COMPACT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayConstructCompact'>>, 'ARRAY_CONTAINS': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayContains'>>, 'ARRAY_HAS': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayContains'>>, 'ARRAY_CONTAINS_ALL': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayContainsAll'>>, 'ARRAY_HAS_ALL': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayContainsAll'>>, 'FILTER': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayFilter'>>, 'ARRAY_FILTER': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayFilter'>>, 'ARRAY_OVERLAPS': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayOverlaps'>>, 'ARRAY_SIZE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArraySize'>>, 'ARRAY_LENGTH': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArraySize'>>, 'ARRAY_SORT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArraySort'>>, 'ARRAY_SUM': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArraySum'>>, 'ARRAY_TO_STRING': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayToString'>>, 'ARRAY_JOIN': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayToString'>>, 'ARRAY_UNION_AGG': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayUnionAgg'>>, 'ARRAY_UNIQUE_AGG': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayUniqueAgg'>>, 'AVG': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Avg'>>, 'CASE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Case'>>, 'CAST': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Cast'>>, 'CAST_TO_STR_TYPE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.CastToStrType'>>, 'CBRT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Cbrt'>>, 'CEIL': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Ceil'>>, 'CEILING': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Ceil'>>, 'CHR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Chr'>>, 'CHAR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Chr'>>, 'COALESCE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Coalesce'>>, 'IFNULL': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Coalesce'>>, 'NVL': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Coalesce'>>, 'COLLATE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Collate'>>, 'COMBINED_AGG_FUNC': <bound method Func.from_arg_list of <class 'sqlglot.expressions.CombinedAggFunc'>>, 'COMBINED_PARAMETERIZED_AGG': <bound method Func.from_arg_list of <class 'sqlglot.expressions.CombinedParameterizedAgg'>>, 'CONCAT': <function Parser.<lambda>>, 'CONCAT_WS': <function Parser.<lambda>>, 'CONNECT_BY_ROOT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ConnectByRoot'>>, 'CONVERT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Convert'>>, 'CORR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Corr'>>, 'COUNT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Count'>>, 'COUNT_IF': <bound method Func.from_arg_list of <class 'sqlglot.expressions.CountIf'>>, 'COUNTIF': <bound method Func.from_arg_list of <class 'sqlglot.expressions.CountIf'>>, 'COVAR_POP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.CovarPop'>>, 'COVAR_SAMP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.CovarSamp'>>, 'CURRENT_DATE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.CurrentDate'>>, 'CURRENT_DATETIME': <bound method Func.from_arg_list of <class 'sqlglot.expressions.CurrentDatetime'>>, 'CURRENT_TIME': <bound method Func.from_arg_list of <class 'sqlglot.expressions.CurrentTime'>>, 'CURRENT_TIMESTAMP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.CurrentTimestamp'>>, 'CURRENT_USER': <bound method Func.from_arg_list of <class 'sqlglot.expressions.CurrentUser'>>, 'DATE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Date'>>, 'DATE_ADD': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DateAdd'>>, 'DATEDIFF': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DateDiff'>>, 'DATE_DIFF': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DateDiff'>>, 'DATE_FROM_PARTS': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DateFromParts'>>, 'DATEFROMPARTS': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DateFromParts'>>, 'DATE_STR_TO_DATE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DateStrToDate'>>, 'DATE_SUB': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DateSub'>>, 'DATE_TO_DATE_STR': <function Parser.<lambda>>, 'DATE_TO_DI': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DateToDi'>>, 'DATE_TRUNC': <function build_timestamp_trunc>, 'DATETIME': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Datetime'>>, 'DATETIME_ADD': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DatetimeAdd'>>, 'DATETIME_DIFF': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DatetimeDiff'>>, 'DATETIME_SUB': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DatetimeSub'>>, 'DATETIME_TRUNC': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DatetimeTrunc'>>, 'DAY': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Day'>>, 'DAY_OF_MONTH': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DayOfMonth'>>, 'DAYOFMONTH': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DayOfMonth'>>, 'DAY_OF_WEEK': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DayOfWeek'>>, 'DAYOFWEEK': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DayOfWeek'>>, 'DAY_OF_YEAR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DayOfYear'>>, 'DAYOFYEAR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DayOfYear'>>, 'DECODE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Decode'>>, 'DI_TO_DATE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DiToDate'>>, 'ENCODE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Encode'>>, 'EXP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Exp'>>, 'EXPLODE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Explode'>>, 'EXPLODE_OUTER': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ExplodeOuter'>>, 'EXTRACT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Extract'>>, 'FIRST': <bound method Func.from_arg_list of <class 'sqlglot.expressions.First'>>, 'FIRST_VALUE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.FirstValue'>>, 'FLATTEN': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Flatten'>>, 'FLOOR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Floor'>>, 'FROM_BASE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.FromBase'>>, 'FROM_BASE64': <bound method Func.from_arg_list of <class 'sqlglot.expressions.FromBase64'>>, 'GAP_FILL': <bound method Func.from_arg_list of <class 'sqlglot.expressions.GapFill'>>, 'GENERATE_DATE_ARRAY': <bound method Func.from_arg_list of <class 'sqlglot.expressions.GenerateDateArray'>>, 'GENERATE_SERIES': <function _build_generate_series>, 'GREATEST': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Greatest'>>, 'GROUP_CONCAT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.GroupConcat'>>, 'HEX': <function build_hex>, 'HLL': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Hll'>>, 'IF': <bound method Func.from_arg_list of <class 'sqlglot.expressions.If'>>, 'IIF': <bound method Func.from_arg_list of <class 'sqlglot.expressions.If'>>, 'INITCAP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Initcap'>>, 'IS_INF': <bound method Func.from_arg_list of <class 'sqlglot.expressions.IsInf'>>, 'ISINF': <bound method Func.from_arg_list of <class 'sqlglot.expressions.IsInf'>>, 'IS_NAN': <bound method Func.from_arg_list of <class 'sqlglot.expressions.IsNan'>>, 'ISNAN': <bound method Func.from_arg_list of <class 'sqlglot.expressions.IsNan'>>, 'J_S_O_N_ARRAY': <bound method Func.from_arg_list of <class 'sqlglot.expressions.JSONArray'>>, 'J_S_O_N_ARRAY_AGG': <bound method Func.from_arg_list of <class 'sqlglot.expressions.JSONArrayAgg'>>, 'JSON_ARRAY_CONTAINS': <bound method Func.from_arg_list of <class 'sqlglot.expressions.JSONArrayContains'>>, 'JSONB_CONTAINS': <bound method Func.from_arg_list of <class 'sqlglot.expressions.JSONBContains'>>, 'JSONB_EXTRACT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.JSONBExtract'>>, 'JSONB_EXTRACT_SCALAR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.JSONBExtractScalar'>>, 'JSON_EXTRACT': <function build_extract_json_with_path.<locals>._builder>, 'JSON_EXTRACT_SCALAR': <function build_extract_json_with_path.<locals>._builder>, 'JSON_FORMAT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.JSONFormat'>>, 'J_S_O_N_OBJECT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.JSONObject'>>, 'J_S_O_N_OBJECT_AGG': <bound method Func.from_arg_list of <class 'sqlglot.expressions.JSONObjectAgg'>>, 'J_S_O_N_TABLE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.JSONTable'>>, 'LAG': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Lag'>>, 'LAST': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Last'>>, 'LAST_DAY': <bound method Func.from_arg_list of <class 'sqlglot.expressions.LastDay'>>, 'LAST_DAY_OF_MONTH': <bound method Func.from_arg_list of <class 'sqlglot.expressions.LastDay'>>, 'LAST_VALUE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.LastValue'>>, 'LEAD': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Lead'>>, 'LEAST': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Least'>>, 'LEFT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Left'>>, 'LENGTH': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Length'>>, 'LEN': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Length'>>, 'LEVENSHTEIN': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Levenshtein'>>, 'LIST': <bound method Func.from_arg_list of <class 'sqlglot.expressions.List'>>, 'LN': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Ln'>>, 'LOG': <function build_logarithm>, 'LOGICAL_AND': <bound method Func.from_arg_list of <class 'sqlglot.expressions.LogicalAnd'>>, 'BOOL_AND': <bound method Func.from_arg_list of <class 'sqlglot.expressions.LogicalAnd'>>, 'BOOLAND_AGG': <bound method Func.from_arg_list of <class 'sqlglot.expressions.LogicalAnd'>>, 'LOGICAL_OR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.LogicalOr'>>, 'BOOL_OR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.LogicalOr'>>, 'BOOLOR_AGG': <bound method Func.from_arg_list of <class 'sqlglot.expressions.LogicalOr'>>, 'LOWER': <function build_lower>, 'LCASE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Lower'>>, 'LOWER_HEX': <bound method Func.from_arg_list of <class 'sqlglot.expressions.LowerHex'>>, 'MD5': <bound method Func.from_arg_list of <class 'sqlglot.expressions.MD5'>>, 'MD5_DIGEST': <bound method Func.from_arg_list of <class 'sqlglot.expressions.MD5Digest'>>, 'MAP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Map'>>, 'MAP_FROM_ENTRIES': <bound method Func.from_arg_list of <class 'sqlglot.expressions.MapFromEntries'>>, 'MATCH_AGAINST': <bound method Func.from_arg_list of <class 'sqlglot.expressions.MatchAgainst'>>, 'MAX': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Max'>>, 'MIN': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Min'>>, 'MONTH': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Month'>>, 'MONTHS_BETWEEN': <bound method Func.from_arg_list of <class 'sqlglot.expressions.MonthsBetween'>>, 'NEXT_VALUE_FOR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.NextValueFor'>>, 'NTH_VALUE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.NthValue'>>, 'NULLIF': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Nullif'>>, 'NUMBER_TO_STR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.NumberToStr'>>, 'NVL2': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Nvl2'>>, 'OBJECT_INSERT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ObjectInsert'>>, 'OPEN_J_S_O_N': <bound method Func.from_arg_list of <class 'sqlglot.expressions.OpenJSON'>>, 'PAD': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Pad'>>, 'PARAMETERIZED_AGG': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ParameterizedAgg'>>, 'PARSE_JSON': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ParseJSON'>>, 'JSON_PARSE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ParseJSON'>>, 'PERCENTILE_CONT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.PercentileCont'>>, 'PERCENTILE_DISC': <bound method Func.from_arg_list of <class 'sqlglot.expressions.PercentileDisc'>>, 'POSEXPLODE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Posexplode'>>, 'POSEXPLODE_OUTER': <bound method Func.from_arg_list of <class 'sqlglot.expressions.PosexplodeOuter'>>, 'POWER': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Pow'>>, 'POW': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Pow'>>, 'PREDICT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Predict'>>, 'QUANTILE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Quantile'>>, 'QUARTER': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Quarter'>>, 'RAND': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Rand'>>, 'RANDOM': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Rand'>>, 'RANDN': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Randn'>>, 'RANGE_N': <bound method Func.from_arg_list of <class 'sqlglot.expressions.RangeN'>>, 'READ_CSV': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ReadCSV'>>, 'REDUCE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Reduce'>>, 'REGEXP_EXTRACT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.RegexpExtract'>>, 'REGEXP_I_LIKE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.RegexpILike'>>, 'REGEXP_LIKE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.RegexpLike'>>, 'REGEXP_REPLACE': <function _build_regexp_replace>, 'REGEXP_SPLIT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.RegexpSplit'>>, 'REPEAT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Repeat'>>, 'RIGHT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Right'>>, 'ROUND': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Round'>>, 'ROW_NUMBER': <bound method Func.from_arg_list of <class 'sqlglot.expressions.RowNumber'>>, 'SHA': <bound method Func.from_arg_list of <class 'sqlglot.expressions.SHA'>>, 'SHA1': <bound method Func.from_arg_list of <class 'sqlglot.expressions.SHA'>>, 'SHA2': <bound method Func.from_arg_list of <class 'sqlglot.expressions.SHA2'>>, 'SAFE_DIVIDE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.SafeDivide'>>, 'SIGN': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Sign'>>, 'SIGNUM': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Sign'>>, 'SORT_ARRAY': <bound method Func.from_arg_list of <class 'sqlglot.expressions.SortArray'>>, 'SPLIT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Split'>>, 'SQRT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Sqrt'>>, 'STANDARD_HASH': <bound method Func.from_arg_list of <class 'sqlglot.expressions.StandardHash'>>, 'STAR_MAP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.StarMap'>>, 'STARTS_WITH': <bound method Func.from_arg_list of <class 'sqlglot.expressions.StartsWith'>>, 'STARTSWITH': <bound method Func.from_arg_list of <class 'sqlglot.expressions.StartsWith'>>, 'STDDEV': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Stddev'>>, 'STDEV': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Stddev'>>, 'STDDEV_POP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.StddevPop'>>, 'STDDEV_SAMP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.StddevSamp'>>, 'STR_POSITION': <bound method Func.from_arg_list of <class 'sqlglot.expressions.StrPosition'>>, 'STR_TO_DATE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.StrToDate'>>, 'STR_TO_MAP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.StrToMap'>>, 'STR_TO_TIME': <bound method Func.from_arg_list of <class 'sqlglot.expressions.StrToTime'>>, 'STR_TO_UNIX': <bound method Func.from_arg_list of <class 'sqlglot.expressions.StrToUnix'>>, 'STRING_TO_ARRAY': <bound method Func.from_arg_list of <class 'sqlglot.expressions.StringToArray'>>, 'SPLIT_BY_STRING': <bound method Func.from_arg_list of <class 'sqlglot.expressions.StringToArray'>>, 'STRUCT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Struct'>>, 'STRUCT_EXTRACT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.StructExtract'>>, 'STUFF': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Stuff'>>, 'INSERT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Stuff'>>, 'SUBSTRING': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Substring'>>, 'SUM': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Sum'>>, 'TIME': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Time'>>, 'TIME_ADD': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimeAdd'>>, 'TIME_DIFF': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimeDiff'>>, 'TIME_FROM_PARTS': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimeFromParts'>>, 'TIMEFROMPARTS': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimeFromParts'>>, 'TIME_STR_TO_DATE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimeStrToDate'>>, 'TIME_STR_TO_TIME': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimeStrToTime'>>, 'TIME_STR_TO_UNIX': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimeStrToUnix'>>, 'TIME_SUB': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimeSub'>>, 'TIME_TO_STR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimeToStr'>>, 'TIME_TO_TIME_STR': <function Parser.<lambda>>, 'TIME_TO_UNIX': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimeToUnix'>>, 'TIME_TRUNC': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimeTrunc'>>, 'TIMESTAMP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Timestamp'>>, 'TIMESTAMP_ADD': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimestampAdd'>>, 'TIMESTAMPDIFF': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimestampDiff'>>, 'TIMESTAMP_DIFF': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimestampDiff'>>, 'TIMESTAMP_FROM_PARTS': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimestampFromParts'>>, 'TIMESTAMPFROMPARTS': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimestampFromParts'>>, 'TIMESTAMP_SUB': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimestampSub'>>, 'TIMESTAMP_TRUNC': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimestampTrunc'>>, 'TO_ARRAY': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ToArray'>>, 'TO_BASE64': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ToBase64'>>, 'TO_CHAR': <function build_formatted_time.<locals>._builder>, 'TO_DAYS': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ToDays'>>, 'TO_MAP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ToMap'>>, 'TO_NUMBER': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ToNumber'>>, 'TRANSFORM': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Transform'>>, 'TRIM': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Trim'>>, 'TRY': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Try'>>, 'TRY_CAST': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TryCast'>>, 'TS_OR_DI_TO_DI': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TsOrDiToDi'>>, 'TS_OR_DS_ADD': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TsOrDsAdd'>>, 'TS_OR_DS_DIFF': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TsOrDsDiff'>>, 'TS_OR_DS_TO_DATE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TsOrDsToDate'>>, 'TS_OR_DS_TO_DATE_STR': <function Parser.<lambda>>, 'TS_OR_DS_TO_TIME': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TsOrDsToTime'>>, 'TS_OR_DS_TO_TIMESTAMP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TsOrDsToTimestamp'>>, 'UNHEX': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Unhex'>>, 'UNIX_DATE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.UnixDate'>>, 'UNIX_TO_STR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.UnixToStr'>>, 'UNIX_TO_TIME': <bound method Func.from_arg_list of <class 'sqlglot.expressions.UnixToTime'>>, 'UNIX_TO_TIME_STR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.UnixToTimeStr'>>, 'UNNEST': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Explode'>>, 'UPPER': <function build_upper>, 'UCASE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Upper'>>, 'VAR_MAP': <function build_var_map>, 'VARIANCE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Variance'>>, 'VARIANCE_SAMP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Variance'>>, 'VAR_SAMP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Variance'>>, 'VARIANCE_POP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.VariancePop'>>, 'VAR_POP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.VariancePop'>>, 'WEEK': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Week'>>, 'WEEK_OF_YEAR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.WeekOfYear'>>, 'WEEKOFYEAR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.WeekOfYear'>>, 'WHEN': <bound method Func.from_arg_list of <class 'sqlglot.expressions.When'>>, 'X_M_L_TABLE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.XMLTable'>>, 'XOR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Xor'>>, 'YEAR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Year'>>, 'GLOB': <function Parser.<lambda>>, 'JSON_EXTRACT_PATH_TEXT': <function build_json_extract_path.<locals>._builder>, 'LIKE': <function build_like>, 'LOG2': <function Parser.<lambda>>, 'LOG10': <function Parser.<lambda>>, 'LPAD': <function Parser.<lambda>>, 'LEFTPAD': <function Parser.<lambda>>, 'MOD': <function build_mod>, 'RPAD': <function Parser.<lambda>>, 'RIGHTPAD': <function Parser.<lambda>>, 'SCOPE_RESOLUTION': <function Parser.<lambda>>, 'TO_HEX': <function build_hex>, 'DIV': <function Postgres.Parser.<lambda>>, 'JSON_EXTRACT_PATH': <function build_json_extract_path.<locals>._builder>, 'MAKE_TIME': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimeFromParts'>>, 'MAKE_TIMESTAMP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimestampFromParts'>>, 'NOW': <bound method Func.from_arg_list of <class 'sqlglot.expressions.CurrentTimestamp'>>, 'TO_DATE': <function build_formatted_time.<locals>._builder>, 'TO_TIMESTAMP': <function _build_to_timestamp>, 'SHA256': <function Postgres.Parser.<lambda>>, 'SHA384': <function Postgres.Parser.<lambda>>, 'SHA512': <function Postgres.Parser.<lambda>>}
FUNCTION_PARSERS = {'CAST': <function Parser.<lambda>>, 'CONVERT': <function Parser.<lambda>>, 'DECODE': <function Parser.<lambda>>, 'EXTRACT': <function Parser.<lambda>>, 'GAP_FILL': <function Parser.<lambda>>, 'JSON_OBJECT': <function Parser.<lambda>>, 'JSON_OBJECTAGG': <function Parser.<lambda>>, 'JSON_TABLE': <function Parser.<lambda>>, 'MATCH': <function Parser.<lambda>>, 'OPENJSON': <function Parser.<lambda>>, 'POSITION': <function Parser.<lambda>>, 'PREDICT': <function Parser.<lambda>>, 'SAFE_CAST': <function Parser.<lambda>>, 'STRING_AGG': <function Parser.<lambda>>, 'SUBSTRING': <function Parser.<lambda>>, 'TRIM': <function Parser.<lambda>>, 'TRY_CAST': <function Parser.<lambda>>, 'TRY_CONVERT': <function Parser.<lambda>>, 'DATE_PART': <function Postgres.Parser.<lambda>>}
BITWISE = {<TokenType.AMP: 'AMP'>: <class 'sqlglot.expressions.BitwiseAnd'>, <TokenType.CARET: 'CARET'>: <class 'sqlglot.expressions.BitwiseXor'>, <TokenType.PIPE: 'PIPE'>: <class 'sqlglot.expressions.BitwiseOr'>, <TokenType.HASH: 'HASH'>: <class 'sqlglot.expressions.BitwiseXor'>}
EXPONENT = {<TokenType.CARET: 'CARET'>: <class 'sqlglot.expressions.Pow'>}
RANGE_PARSERS = {<TokenType.BETWEEN: 'BETWEEN'>: <function Parser.<lambda>>, <TokenType.GLOB: 'GLOB'>: <function binary_range_parser.<locals>._parse_binary_range>, <TokenType.ILIKE: 'ILIKE'>: <function binary_range_parser.<locals>._parse_binary_range>, <TokenType.IN: 'IN'>: <function Parser.<lambda>>, <TokenType.IRLIKE: 'IRLIKE'>: <function binary_range_parser.<locals>._parse_binary_range>, <TokenType.IS: 'IS'>: <function Parser.<lambda>>, <TokenType.LIKE: 'LIKE'>: <function binary_range_parser.<locals>._parse_binary_range>, <TokenType.OVERLAPS: 'OVERLAPS'>: <function binary_range_parser.<locals>._parse_binary_range>, <TokenType.RLIKE: 'RLIKE'>: <function binary_range_parser.<locals>._parse_binary_range>, <TokenType.SIMILAR_TO: 'SIMILAR_TO'>: <function binary_range_parser.<locals>._parse_binary_range>, <TokenType.FOR: 'FOR'>: <function Parser.<lambda>>, <TokenType.AT_GT: 'AT_GT'>: <function binary_range_parser.<locals>._parse_binary_range>, <TokenType.DAMP: 'DAMP'>: <function binary_range_parser.<locals>._parse_binary_range>, <TokenType.DAT: 'DAT'>: <function Postgres.Parser.<lambda>>, <TokenType.LT_AT: 'LT_AT'>: <function binary_range_parser.<locals>._parse_binary_range>, <TokenType.OPERATOR: 'OPERATOR'>: <function Postgres.Parser.<lambda>>}
STATEMENT_PARSERS = {<TokenType.ALTER: 'ALTER'>: <function Parser.<lambda>>, <TokenType.BEGIN: 'BEGIN'>: <function Parser.<lambda>>, <TokenType.CACHE: 'CACHE'>: <function Parser.<lambda>>, <TokenType.COMMENT: 'COMMENT'>: <function Parser.<lambda>>, <TokenType.COMMIT: 'COMMIT'>: <function Parser.<lambda>>, <TokenType.COPY: 'COPY'>: <function Parser.<lambda>>, <TokenType.CREATE: 'CREATE'>: <function Parser.<lambda>>, <TokenType.DELETE: 'DELETE'>: <function Parser.<lambda>>, <TokenType.DESC: 'DESC'>: <function Parser.<lambda>>, <TokenType.DESCRIBE: 'DESCRIBE'>: <function Parser.<lambda>>, <TokenType.DROP: 'DROP'>: <function Parser.<lambda>>, <TokenType.INSERT: 'INSERT'>: <function Parser.<lambda>>, <TokenType.KILL: 'KILL'>: <function Parser.<lambda>>, <TokenType.LOAD: 'LOAD'>: <function Parser.<lambda>>, <TokenType.MERGE: 'MERGE'>: <function Parser.<lambda>>, <TokenType.PIVOT: 'PIVOT'>: <function Parser.<lambda>>, <TokenType.PRAGMA: 'PRAGMA'>: <function Parser.<lambda>>, <TokenType.REFRESH: 'REFRESH'>: <function Parser.<lambda>>, <TokenType.ROLLBACK: 'ROLLBACK'>: <function Parser.<lambda>>, <TokenType.SET: 'SET'>: <function Parser.<lambda>>, <TokenType.TRUNCATE: 'TRUNCATE'>: <function Parser.<lambda>>, <TokenType.UNCACHE: 'UNCACHE'>: <function Parser.<lambda>>, <TokenType.UPDATE: 'UPDATE'>: <function Parser.<lambda>>, <TokenType.USE: 'USE'>: <function Parser.<lambda>>, <TokenType.SEMICOLON: 'SEMICOLON'>: <function Parser.<lambda>>, <TokenType.END: 'END'>: <function Postgres.Parser.<lambda>>}
JSON_ARROWS_REQUIRE_JSON_TYPE = True
COLUMN_OPERATORS = {<TokenType.DOT: 'DOT'>: None, <TokenType.DCOLON: 'DCOLON'>: <function Parser.<lambda>>, <TokenType.ARROW: 'ARROW'>: <function Postgres.Parser.<lambda>>, <TokenType.DARROW: 'DARROW'>: <function Postgres.Parser.<lambda>>, <TokenType.HASH_ARROW: 'HASH_ARROW'>: <function Parser.<lambda>>, <TokenType.DHASH_ARROW: 'DHASH_ARROW'>: <function Parser.<lambda>>, <TokenType.PLACEHOLDER: 'PLACEHOLDER'>: <function Parser.<lambda>>}
ID_VAR_TOKENS = {<TokenType.TIMESTAMPNTZ: 'TIMESTAMPNTZ'>, <TokenType.FINAL: 'FINAL'>, <TokenType.MEDIUMINT: 'MEDIUMINT'>, <TokenType.DATERANGE: 'DATERANGE'>, <TokenType.TAG: 'TAG'>, <TokenType.GEOGRAPHY: 'GEOGRAPHY'>, <TokenType.TIME: 'TIME'>, <TokenType.COPY: 'COPY'>, <TokenType.DELETE: 'DELETE'>, <TokenType.ASOF: 'ASOF'>, <TokenType.INT256: 'INT256'>, <TokenType.TRUE: 'TRUE'>, <TokenType.FLOAT: 'FLOAT'>, <TokenType.ANTI: 'ANTI'>, <TokenType.TINYINT: 'TINYINT'>, <TokenType.TIMESTAMP_S: 'TIMESTAMP_S'>, <TokenType.FILTER: 'FILTER'>, <TokenType.CACHE: 'CACHE'>, <TokenType.VECTOR: 'VECTOR'>, <TokenType.OBJECT_IDENTIFIER: 'OBJECT_IDENTIFIER'>, <TokenType.ROWS: 'ROWS'>, <TokenType.NESTED: 'NESTED'>, <TokenType.CURRENT_DATETIME: 'CURRENT_DATETIME'>, <TokenType.KEEP: 'KEEP'>, <TokenType.CASE: 'CASE'>, <TokenType.KILL: 'KILL'>, <TokenType.TEMPORARY: 'TEMPORARY'>, <TokenType.UINT128: 'UINT128'>, <TokenType.COMMIT: 'COMMIT'>, <TokenType.ROWVERSION: 'ROWVERSION'>, <TokenType.IDENTIFIER: 'IDENTIFIER'>, <TokenType.STREAMLIT: 'STREAMLIT'>, <TokenType.IPV6: 'IPV6'>, <TokenType.GEOMETRY: 'GEOMETRY'>, <TokenType.COLUMN: 'COLUMN'>, <TokenType.TINYTEXT: 'TINYTEXT'>, <TokenType.WINDOW: 'WINDOW'>, <TokenType.BINARY: 'BINARY'>, <TokenType.BOOLEAN: 'BOOLEAN'>, <TokenType.ROW: 'ROW'>, <TokenType.TEXT: 'TEXT'>, <TokenType.IPPREFIX: 'IPPREFIX'>, <TokenType.DICTIONARY: 'DICTIONARY'>, <TokenType.ISNULL: 'ISNULL'>, <TokenType.PERCENT: 'PERCENT'>, <TokenType.SCHEMA: 'SCHEMA'>, <TokenType.ENUM16: 'ENUM16'>, <TokenType.UTINYINT: 'UTINYINT'>, <TokenType.CURRENT_TIME: 'CURRENT_TIME'>, <TokenType.USE: 'USE'>, <TokenType.INDEX: 'INDEX'>, <TokenType.IPV4: 'IPV4'>, <TokenType.INT128: 'INT128'>, <TokenType.EXECUTE: 'EXECUTE'>, <TokenType.TABLE: 'TABLE'>, <TokenType.UNPIVOT: 'UNPIVOT'>, <TokenType.DECIMAL: 'DECIMAL'>, <TokenType.LONGTEXT: 'LONGTEXT'>, <TokenType.TIMESTAMP_MS: 'TIMESTAMP_MS'>, <TokenType.TINYBLOB: 'TINYBLOB'>, <TokenType.CONSTRAINT: 'CONSTRAINT'>, <TokenType.NUMRANGE: 'NUMRANGE'>, <TokenType.SETTINGS: 'SETTINGS'>, <TokenType.NULLABLE: 'NULLABLE'>, <TokenType.STRAIGHT_JOIN: 'STRAIGHT_JOIN'>, <TokenType.IMAGE: 'IMAGE'>, <TokenType.TSTZMULTIRANGE: 'TSTZMULTIRANGE'>, <TokenType.LIST: 'LIST'>, <TokenType.INT4MULTIRANGE: 'INT4MULTIRANGE'>, <TokenType.SEQUENCE: 'SEQUENCE'>, <TokenType.ALL: 'ALL'>, <TokenType.MAP: 'MAP'>, <TokenType.OPERATOR: 'OPERATOR'>, <TokenType.LOWCARDINALITY: 'LOWCARDINALITY'>, <TokenType.UNKNOWN: 'UNKNOWN'>, <TokenType.NULL: 'NULL'>, <TokenType.LEFT: 'LEFT'>, <TokenType.SEMI: 'SEMI'>, <TokenType.ROLLUP: 'ROLLUP'>, <TokenType.INT4RANGE: 'INT4RANGE'>, <TokenType.TDIGEST: 'TDIGEST'>, <TokenType.CURRENT_TIMESTAMP: 'CURRENT_TIMESTAMP'>, <TokenType.MONEY: 'MONEY'>, <TokenType.TIMESTAMP_NS: 'TIMESTAMP_NS'>, <TokenType.STRUCT: 'STRUCT'>, <TokenType.NATURAL: 'NATURAL'>, <TokenType.SMALLINT: 'SMALLINT'>, <TokenType.VIEW: 'VIEW'>, <TokenType.SMALLMONEY: 'SMALLMONEY'>, <TokenType.SIMPLEAGGREGATEFUNCTION: 'SIMPLEAGGREGATEFUNCTION'>, <TokenType.VARCHAR: 'VARCHAR'>, <TokenType.COLLATE: 'COLLATE'>, <TokenType.NEXT: 'NEXT'>, <TokenType.PARTITION: 'PARTITION'>, <TokenType.OVERLAPS: 'OVERLAPS'>, <TokenType.NCHAR: 'NCHAR'>, <TokenType.MEDIUMTEXT: 'MEDIUMTEXT'>, <TokenType.OFFSET: 'OFFSET'>, <TokenType.ORDINALITY: 'ORDINALITY'>, <TokenType.BPCHAR: 'BPCHAR'>, <TokenType.SUPER: 'SUPER'>, <TokenType.END: 'END'>, <TokenType.USMALLINT: 'USMALLINT'>, <TokenType.TSTZRANGE: 'TSTZRANGE'>, <TokenType.COMMENT: 'COMMENT'>, <TokenType.UPDATE: 'UPDATE'>, <TokenType.EXISTS: 'EXISTS'>, <TokenType.CHAR: 'CHAR'>, <TokenType.WAREHOUSE: 'WAREHOUSE'>, <TokenType.ESCAPE: 'ESCAPE'>, <TokenType.LONGBLOB: 'LONGBLOB'>, <TokenType.CURRENT_USER: 'CURRENT_USER'>, <TokenType.UINT: 'UINT'>, <TokenType.VARIANT: 'VARIANT'>, <TokenType.ENUM8: 'ENUM8'>, <TokenType.BIGINT: 'BIGINT'>, <TokenType.TOP: 'TOP'>, <TokenType.TIMESTAMP: 'TIMESTAMP'>, <TokenType.SOME: 'SOME'>, <TokenType.UMEDIUMINT: 'UMEDIUMINT'>, <TokenType.DATEMULTIRANGE: 'DATEMULTIRANGE'>, <TokenType.USERDEFINED: 'USERDEFINED'>, <TokenType.DESC: 'DESC'>, <TokenType.NUMMULTIRANGE: 'NUMMULTIRANGE'>, <TokenType.RANGE: 'RANGE'>, <TokenType.FIRST: 'FIRST'>, <TokenType.UUID: 'UUID'>, <TokenType.DESCRIBE: 'DESCRIBE'>, <TokenType.MODEL: 'MODEL'>, <TokenType.AGGREGATEFUNCTION: 'AGGREGATEFUNCTION'>, <TokenType.MERGE: 'MERGE'>, <TokenType.INT8MULTIRANGE: 'INT8MULTIRANGE'>, <TokenType.RECURSIVE: 'RECURSIVE'>, <TokenType.VAR: 'VAR'>, <TokenType.NVARCHAR: 'NVARCHAR'>, <TokenType.NAME: 'NAME'>, <TokenType.JSON: 'JSON'>, <TokenType.FULL: 'FULL'>, <TokenType.BEGIN: 'BEGIN'>, <TokenType.IS: 'IS'>, <TokenType.TSMULTIRANGE: 'TSMULTIRANGE'>, <TokenType.YEAR: 'YEAR'>, <TokenType.XML: 'XML'>, <TokenType.OBJECT: 'OBJECT'>, <TokenType.DATE32: 'DATE32'>, <TokenType.DATETIME64: 'DATETIME64'>, <TokenType.DATETIME: 'DATETIME'>, <TokenType.JSONB: 'JSONB'>, <TokenType.PIVOT: 'PIVOT'>, <TokenType.BIT: 'BIT'>, <TokenType.FORMAT: 'FORMAT'>, <TokenType.PRAGMA: 'PRAGMA'>, <TokenType.FOREIGN_KEY: 'FOREIGN_KEY'>, <TokenType.UNIQUEIDENTIFIER: 'UNIQUEIDENTIFIER'>, <TokenType.APPLY: 'APPLY'>, <TokenType.TSRANGE: 'TSRANGE'>, <TokenType.LOAD: 'LOAD'>, <TokenType.RIGHT: 'RIGHT'>, <TokenType.BIGSERIAL: 'BIGSERIAL'>, <TokenType.VARBINARY: 'VARBINARY'>, <TokenType.UINT256: 'UINT256'>, <TokenType.DIV: 'DIV'>, <TokenType.ARRAY: 'ARRAY'>, <TokenType.SHOW: 'SHOW'>, <TokenType.ASC: 'ASC'>, <TokenType.INT: 'INT'>, <TokenType.IPADDRESS: 'IPADDRESS'>, <TokenType.INET: 'INET'>, <TokenType.INT8RANGE: 'INT8RANGE'>, <TokenType.SMALLSERIAL: 'SMALLSERIAL'>, <TokenType.DEFAULT: 'DEFAULT'>, <TokenType.ANY: 'ANY'>, <TokenType.HLLSKETCH: 'HLLSKETCH'>, <TokenType.VOLATILE: 'VOLATILE'>, <TokenType.TIMESTAMPTZ: 'TIMESTAMPTZ'>, <TokenType.FIXEDSTRING: 'FIXEDSTRING'>, <TokenType.TRUNCATE: 'TRUNCATE'>, <TokenType.DOUBLE: 'DOUBLE'>, <TokenType.UBIGINT: 'UBIGINT'>, <TokenType.FUNCTION: 'FUNCTION'>, <TokenType.UDECIMAL: 'UDECIMAL'>, <TokenType.REPLACE: 'REPLACE'>, <TokenType.OVERWRITE: 'OVERWRITE'>, <TokenType.REFERENCES: 'REFERENCES'>, <TokenType.HSTORE: 'HSTORE'>, <TokenType.DATABASE: 'DATABASE'>, <TokenType.FALSE: 'FALSE'>, <TokenType.UNNEST: 'UNNEST'>, <TokenType.ENUM: 'ENUM'>, <TokenType.REFRESH: 'REFRESH'>, <TokenType.TIMESTAMPLTZ: 'TIMESTAMPLTZ'>, <TokenType.COMMAND: 'COMMAND'>, <TokenType.INTERVAL: 'INTERVAL'>, <TokenType.BIGDECIMAL: 'BIGDECIMAL'>, <TokenType.CURRENT_DATE: 'CURRENT_DATE'>, <TokenType.UNIQUE: 'UNIQUE'>, <TokenType.PSEUDO_TYPE: 'PSEUDO_TYPE'>, <TokenType.MEDIUMBLOB: 'MEDIUMBLOB'>, <TokenType.AUTO_INCREMENT: 'AUTO_INCREMENT'>, <TokenType.TIMETZ: 'TIMETZ'>, <TokenType.SET: 'SET'>, <TokenType.SERIAL: 'SERIAL'>, <TokenType.DATE: 'DATE'>, <TokenType.STORAGE_INTEGRATION: 'STORAGE_INTEGRATION'>, <TokenType.PROCEDURE: 'PROCEDURE'>}
TABLE_ALIAS_TOKENS = {<TokenType.TIMESTAMPNTZ: 'TIMESTAMPNTZ'>, <TokenType.FINAL: 'FINAL'>, <TokenType.MEDIUMINT: 'MEDIUMINT'>, <TokenType.DATERANGE: 'DATERANGE'>, <TokenType.TAG: 'TAG'>, <TokenType.GEOGRAPHY: 'GEOGRAPHY'>, <TokenType.TIME: 'TIME'>, <TokenType.COPY: 'COPY'>, <TokenType.DELETE: 'DELETE'>, <TokenType.INT256: 'INT256'>, <TokenType.TRUE: 'TRUE'>, <TokenType.FLOAT: 'FLOAT'>, <TokenType.TINYINT: 'TINYINT'>, <TokenType.TIMESTAMP_S: 'TIMESTAMP_S'>, <TokenType.FILTER: 'FILTER'>, <TokenType.CACHE: 'CACHE'>, <TokenType.VECTOR: 'VECTOR'>, <TokenType.OBJECT_IDENTIFIER: 'OBJECT_IDENTIFIER'>, <TokenType.ROWS: 'ROWS'>, <TokenType.NESTED: 'NESTED'>, <TokenType.CURRENT_DATETIME: 'CURRENT_DATETIME'>, <TokenType.KEEP: 'KEEP'>, <TokenType.CASE: 'CASE'>, <TokenType.KILL: 'KILL'>, <TokenType.TEMPORARY: 'TEMPORARY'>, <TokenType.UINT128: 'UINT128'>, <TokenType.COMMIT: 'COMMIT'>, <TokenType.ROWVERSION: 'ROWVERSION'>, <TokenType.IDENTIFIER: 'IDENTIFIER'>, <TokenType.STREAMLIT: 'STREAMLIT'>, <TokenType.IPV6: 'IPV6'>, <TokenType.GEOMETRY: 'GEOMETRY'>, <TokenType.COLUMN: 'COLUMN'>, <TokenType.TINYTEXT: 'TINYTEXT'>, <TokenType.BINARY: 'BINARY'>, <TokenType.BOOLEAN: 'BOOLEAN'>, <TokenType.ROW: 'ROW'>, <TokenType.TEXT: 'TEXT'>, <TokenType.IPPREFIX: 'IPPREFIX'>, <TokenType.DICTIONARY: 'DICTIONARY'>, <TokenType.ISNULL: 'ISNULL'>, <TokenType.PERCENT: 'PERCENT'>, <TokenType.SCHEMA: 'SCHEMA'>, <TokenType.ENUM16: 'ENUM16'>, <TokenType.UTINYINT: 'UTINYINT'>, <TokenType.CURRENT_TIME: 'CURRENT_TIME'>, <TokenType.USE: 'USE'>, <TokenType.INDEX: 'INDEX'>, <TokenType.IPV4: 'IPV4'>, <TokenType.INT128: 'INT128'>, <TokenType.EXECUTE: 'EXECUTE'>, <TokenType.TABLE: 'TABLE'>, <TokenType.UNPIVOT: 'UNPIVOT'>, <TokenType.DECIMAL: 'DECIMAL'>, <TokenType.LONGTEXT: 'LONGTEXT'>, <TokenType.TIMESTAMP_MS: 'TIMESTAMP_MS'>, <TokenType.TINYBLOB: 'TINYBLOB'>, <TokenType.CONSTRAINT: 'CONSTRAINT'>, <TokenType.NUMRANGE: 'NUMRANGE'>, <TokenType.SETTINGS: 'SETTINGS'>, <TokenType.NULLABLE: 'NULLABLE'>, <TokenType.STRAIGHT_JOIN: 'STRAIGHT_JOIN'>, <TokenType.IMAGE: 'IMAGE'>, <TokenType.TSTZMULTIRANGE: 'TSTZMULTIRANGE'>, <TokenType.LIST: 'LIST'>, <TokenType.INT4MULTIRANGE: 'INT4MULTIRANGE'>, <TokenType.SEQUENCE: 'SEQUENCE'>, <TokenType.ALL: 'ALL'>, <TokenType.MAP: 'MAP'>, <TokenType.OPERATOR: 'OPERATOR'>, <TokenType.LOWCARDINALITY: 'LOWCARDINALITY'>, <TokenType.UNKNOWN: 'UNKNOWN'>, <TokenType.NULL: 'NULL'>, <TokenType.ROLLUP: 'ROLLUP'>, <TokenType.INT4RANGE: 'INT4RANGE'>, <TokenType.TDIGEST: 'TDIGEST'>, <TokenType.CURRENT_TIMESTAMP: 'CURRENT_TIMESTAMP'>, <TokenType.MONEY: 'MONEY'>, <TokenType.TIMESTAMP_NS: 'TIMESTAMP_NS'>, <TokenType.STRUCT: 'STRUCT'>, <TokenType.SMALLINT: 'SMALLINT'>, <TokenType.VIEW: 'VIEW'>, <TokenType.SMALLMONEY: 'SMALLMONEY'>, <TokenType.SIMPLEAGGREGATEFUNCTION: 'SIMPLEAGGREGATEFUNCTION'>, <TokenType.VARCHAR: 'VARCHAR'>, <TokenType.COLLATE: 'COLLATE'>, <TokenType.NEXT: 'NEXT'>, <TokenType.PARTITION: 'PARTITION'>, <TokenType.OVERLAPS: 'OVERLAPS'>, <TokenType.NCHAR: 'NCHAR'>, <TokenType.MEDIUMTEXT: 'MEDIUMTEXT'>, <TokenType.ORDINALITY: 'ORDINALITY'>, <TokenType.BPCHAR: 'BPCHAR'>, <TokenType.SUPER: 'SUPER'>, <TokenType.END: 'END'>, <TokenType.USMALLINT: 'USMALLINT'>, <TokenType.TSTZRANGE: 'TSTZRANGE'>, <TokenType.COMMENT: 'COMMENT'>, <TokenType.UPDATE: 'UPDATE'>, <TokenType.EXISTS: 'EXISTS'>, <TokenType.CHAR: 'CHAR'>, <TokenType.WAREHOUSE: 'WAREHOUSE'>, <TokenType.ESCAPE: 'ESCAPE'>, <TokenType.LONGBLOB: 'LONGBLOB'>, <TokenType.CURRENT_USER: 'CURRENT_USER'>, <TokenType.UINT: 'UINT'>, <TokenType.VARIANT: 'VARIANT'>, <TokenType.ENUM8: 'ENUM8'>, <TokenType.BIGINT: 'BIGINT'>, <TokenType.TOP: 'TOP'>, <TokenType.TIMESTAMP: 'TIMESTAMP'>, <TokenType.SOME: 'SOME'>, <TokenType.UMEDIUMINT: 'UMEDIUMINT'>, <TokenType.DATEMULTIRANGE: 'DATEMULTIRANGE'>, <TokenType.USERDEFINED: 'USERDEFINED'>, <TokenType.DESC: 'DESC'>, <TokenType.NUMMULTIRANGE: 'NUMMULTIRANGE'>, <TokenType.RANGE: 'RANGE'>, <TokenType.FIRST: 'FIRST'>, <TokenType.UUID: 'UUID'>, <TokenType.DESCRIBE: 'DESCRIBE'>, <TokenType.MODEL: 'MODEL'>, <TokenType.AGGREGATEFUNCTION: 'AGGREGATEFUNCTION'>, <TokenType.MERGE: 'MERGE'>, <TokenType.INT8MULTIRANGE: 'INT8MULTIRANGE'>, <TokenType.RECURSIVE: 'RECURSIVE'>, <TokenType.VAR: 'VAR'>, <TokenType.NVARCHAR: 'NVARCHAR'>, <TokenType.NAME: 'NAME'>, <TokenType.JSON: 'JSON'>, <TokenType.BEGIN: 'BEGIN'>, <TokenType.IS: 'IS'>, <TokenType.TSMULTIRANGE: 'TSMULTIRANGE'>, <TokenType.YEAR: 'YEAR'>, <TokenType.XML: 'XML'>, <TokenType.OBJECT: 'OBJECT'>, <TokenType.DATE32: 'DATE32'>, <TokenType.DATETIME64: 'DATETIME64'>, <TokenType.DATETIME: 'DATETIME'>, <TokenType.JSONB: 'JSONB'>, <TokenType.PIVOT: 'PIVOT'>, <TokenType.BIT: 'BIT'>, <TokenType.FORMAT: 'FORMAT'>, <TokenType.PRAGMA: 'PRAGMA'>, <TokenType.FOREIGN_KEY: 'FOREIGN_KEY'>, <TokenType.UNIQUEIDENTIFIER: 'UNIQUEIDENTIFIER'>, <TokenType.TSRANGE: 'TSRANGE'>, <TokenType.LOAD: 'LOAD'>, <TokenType.BIGSERIAL: 'BIGSERIAL'>, <TokenType.VARBINARY: 'VARBINARY'>, <TokenType.UINT256: 'UINT256'>, <TokenType.DIV: 'DIV'>, <TokenType.ARRAY: 'ARRAY'>, <TokenType.SHOW: 'SHOW'>, <TokenType.ASC: 'ASC'>, <TokenType.INT: 'INT'>, <TokenType.IPADDRESS: 'IPADDRESS'>, <TokenType.INET: 'INET'>, <TokenType.INT8RANGE: 'INT8RANGE'>, <TokenType.SMALLSERIAL: 'SMALLSERIAL'>, <TokenType.PROCEDURE: 'PROCEDURE'>, <TokenType.DEFAULT: 'DEFAULT'>, <TokenType.ANY: 'ANY'>, <TokenType.HLLSKETCH: 'HLLSKETCH'>, <TokenType.VOLATILE: 'VOLATILE'>, <TokenType.TIMESTAMPTZ: 'TIMESTAMPTZ'>, <TokenType.FIXEDSTRING: 'FIXEDSTRING'>, <TokenType.TRUNCATE: 'TRUNCATE'>, <TokenType.DOUBLE: 'DOUBLE'>, <TokenType.UBIGINT: 'UBIGINT'>, <TokenType.FUNCTION: 'FUNCTION'>, <TokenType.UDECIMAL: 'UDECIMAL'>, <TokenType.REPLACE: 'REPLACE'>, <TokenType.OVERWRITE: 'OVERWRITE'>, <TokenType.REFERENCES: 'REFERENCES'>, <TokenType.HSTORE: 'HSTORE'>, <TokenType.DATABASE: 'DATABASE'>, <TokenType.FALSE: 'FALSE'>, <TokenType.UNNEST: 'UNNEST'>, <TokenType.ENUM: 'ENUM'>, <TokenType.REFRESH: 'REFRESH'>, <TokenType.TIMESTAMPLTZ: 'TIMESTAMPLTZ'>, <TokenType.COMMAND: 'COMMAND'>, <TokenType.INTERVAL: 'INTERVAL'>, <TokenType.CURRENT_DATE: 'CURRENT_DATE'>, <TokenType.UNIQUE: 'UNIQUE'>, <TokenType.PSEUDO_TYPE: 'PSEUDO_TYPE'>, <TokenType.MEDIUMBLOB: 'MEDIUMBLOB'>, <TokenType.AUTO_INCREMENT: 'AUTO_INCREMENT'>, <TokenType.TIMETZ: 'TIMETZ'>, <TokenType.SET: 'SET'>, <TokenType.SERIAL: 'SERIAL'>, <TokenType.DATE: 'DATE'>, <TokenType.STORAGE_INTEGRATION: 'STORAGE_INTEGRATION'>, <TokenType.BIGDECIMAL: 'BIGDECIMAL'>}
SHOW_TRIE: Dict = {}
SET_TRIE: Dict = {'GLOBAL': {0: True}, 'LOCAL': {0: True}, 'SESSION': {0: True}, 'TRANSACTION': {0: True}}

##### Inherited Members

sqlglot.parser.Parser
  Parser
  NO_PAREN_FUNCTIONS
  STRUCT_TYPE_TOKENS
  NESTED_TYPE_TOKENS
  ENUM_TYPE_TOKENS
  AGGREGATE_TYPE_TOKENS
  TYPE_TOKENS
  SIGNED_TO_UNSIGNED_TYPE_TOKEN
  SUBQUERY_PREDICATES
  RESERVED_TOKENS
  DB_CREATABLES
  CREATABLES
  INTERVAL_VARS
  ALIAS_TOKENS
  ARRAY_CONSTRUCTORS
  COMMENT_TABLE_ALIAS_TOKENS
  UPDATE_ALIAS_TOKENS
  TRIM_TYPES
  FUNC_TOKENS
  CONJUNCTION
  ASSIGNMENT
  DISJUNCTION
  EQUALITY
  COMPARISON
  TERM
  FACTOR
  TIMES
  TIMESTAMPS
  SET_OPERATIONS
  JOIN_METHODS
  JOIN_SIDES
  JOIN_KINDS
  JOIN_HINTS
  LAMBDAS
  EXPRESSION_PARSERS
  UNARY_PARSERS
  STRING_PARSERS
  NUMERIC_PARSERS
  PRIMARY_PARSERS
  PLACEHOLDER_PARSERS
  CONSTRAINT_PARSERS
  ALTER_PARSERS
  ALTER_ALTER_PARSERS
  SCHEMA_UNNAMED_CONSTRAINTS
  NO_PAREN_FUNCTION_PARSERS
  INVALID_FUNC_NAME_TOKENS
  FUNCTIONS_WITH_ALIASED_ARGS
  KEY_VALUE_DEFINITIONS
  QUERY_MODIFIER_PARSERS
  SET_PARSERS
  SHOW_PARSERS
  TYPE_LITERAL_PARSERS
  TYPE_CONVERTERS
  DDL_SELECT_TOKENS
  PRE_VOLATILE_TOKENS
  TRANSACTION_KIND
  TRANSACTION_CHARACTERISTICS
  CONFLICT_ACTIONS
  CREATE_SEQUENCE
  ISOLATED_LOADING_OPTIONS
  USABLES
  CAST_ACTIONS
  SCHEMA_BINDING_OPTIONS
  KEY_CONSTRAINT_OPTIONS
  INSERT_ALTERNATIVES
  CLONE_KEYWORDS
  HISTORICAL_DATA_PREFIX
  HISTORICAL_DATA_KIND
  OPCLASS_FOLLOW_KEYWORDS
  OPTYPE_FOLLOW_TOKENS
  TABLE_INDEX_HINT_TOKENS
  VIEW_ATTRIBUTES
  WINDOW_ALIAS_TOKENS
  WINDOW_BEFORE_PAREN_TOKENS
  WINDOW_SIDES
  JSON_KEY_VALUE_SEPARATOR_TOKENS
  FETCH_TOKENS
  ADD_CONSTRAINT_TOKENS
  DISTINCT_TOKENS
  NULL_TOKENS
  UNNEST_OFFSET_ALIAS_TOKENS
  SELECT_START_TOKENS
  COPY_INTO_VARLEN_OPTIONS
  STRICT_CAST
  PREFIXED_PIVOT_COLUMNS
  IDENTIFY_PIVOT_STRINGS
  LOG_DEFAULTS_TO_LN
  ALTER_TABLE_ADD_REQUIRED_FOR_EACH_COLUMN
  TABLESAMPLE_CSV
  DEFAULT_SAMPLING_METHOD
  SET_REQUIRES_ASSIGNMENT_DELIMITER
  TRIM_PATTERN_FIRST
  STRING_ALIASES
  MODIFIERS_ATTACHED_TO_SET_OP
  SET_OP_MODIFIERS
  NO_PAREN_IF_COMMANDS
  COLON_IS_VARIANT_EXTRACT
  VALUES_FOLLOWED_BY_PAREN
  SUPPORTS_IMPLICIT_UNNEST
  INTERVAL_SPANS
  SUPPORTS_PARTITION_SELECTION
  error_level
  error_message_context
  max_errors
  dialect
  reset
  parse
  parse_into
  check_errors
  raise_error
  expression
  validate_expression
  errors
  sql

class Postgres.Generator(sqlglot.generator.Generator): View Source

    455    class Generator(generator.Generator):
    456        SINGLE_STRING_INTERVAL = True
    457        RENAME_TABLE_WITH_DB = False
    458        LOCKING_READS_SUPPORTED = True
    459        JOIN_HINTS = False
    460        TABLE_HINTS = False
    461        QUERY_HINTS = False
    462        NVL2_SUPPORTED = False
    463        PARAMETER_TOKEN = "$"
    464        TABLESAMPLE_SIZE_IS_ROWS = False
    465        TABLESAMPLE_SEED_KEYWORD = "REPEATABLE"
    466        SUPPORTS_SELECT_INTO = True
    467        JSON_TYPE_REQUIRED_FOR_EXTRACTION = True
    468        SUPPORTS_UNLOGGED_TABLES = True
    469        LIKE_PROPERTY_INSIDE_SCHEMA = True
    470        MULTI_ARG_DISTINCT = False
    471        CAN_IMPLEMENT_ARRAY_ANY = True
    472        COPY_HAS_INTO_KEYWORD = False
    473
    474        SUPPORTED_JSON_PATH_PARTS = {
    475            exp.JSONPathKey,
    476            exp.JSONPathRoot,
    477            exp.JSONPathSubscript,
    478        }
    479
    480        TYPE_MAPPING = {
    481            **generator.Generator.TYPE_MAPPING,
    482            exp.DataType.Type.TINYINT: "SMALLINT",
    483            exp.DataType.Type.FLOAT: "REAL",
    484            exp.DataType.Type.DOUBLE: "DOUBLE PRECISION",
    485            exp.DataType.Type.BINARY: "BYTEA",
    486            exp.DataType.Type.VARBINARY: "BYTEA",
    487            exp.DataType.Type.ROWVERSION: "BYTEA",
    488            exp.DataType.Type.DATETIME: "TIMESTAMP",
    489        }
    490
    491        TRANSFORMS = {
    492            **generator.Generator.TRANSFORMS,
    493            exp.AnyValue: any_value_to_max_sql,
    494            exp.Array: lambda self, e: (
    495                f"{self.normalize_func('ARRAY')}({self.sql(e.expressions[0])})"
    496                if isinstance(seq_get(e.expressions, 0), exp.Select)
    497                else f"{self.normalize_func('ARRAY')}[{self.expressions(e, flat=True)}]"
    498            ),
    499            exp.ArrayConcat: rename_func("ARRAY_CAT"),
    500            exp.ArrayContainsAll: lambda self, e: self.binary(e, "@>"),
    501            exp.ArrayOverlaps: lambda self, e: self.binary(e, "&&"),
    502            exp.ArrayFilter: filter_array_using_unnest,
    503            exp.ArraySize: lambda self, e: self.func("ARRAY_LENGTH", e.this, e.expression or "1"),
    504            exp.BitwiseXor: lambda self, e: self.binary(e, "#"),
    505            exp.ColumnDef: transforms.preprocess([_auto_increment_to_serial, _serial_to_generated]),
    506            exp.CurrentDate: no_paren_current_date_sql,
    507            exp.CurrentTimestamp: lambda *_: "CURRENT_TIMESTAMP",
    508            exp.CurrentUser: lambda *_: "CURRENT_USER",
    509            exp.DateAdd: _date_add_sql("+"),
    510            exp.DateDiff: _date_diff_sql,
    511            exp.DateStrToDate: datestrtodate_sql,
    512            exp.DateSub: _date_add_sql("-"),
    513            exp.Explode: rename_func("UNNEST"),
    514            exp.GroupConcat: _string_agg_sql,
    515            exp.IntDiv: rename_func("DIV"),
    516            exp.JSONExtract: _json_extract_sql("JSON_EXTRACT_PATH", "->"),
    517            exp.JSONExtractScalar: _json_extract_sql("JSON_EXTRACT_PATH_TEXT", "->>"),
    518            exp.JSONBExtract: lambda self, e: self.binary(e, "#>"),
    519            exp.JSONBExtractScalar: lambda self, e: self.binary(e, "#>>"),
    520            exp.JSONBContains: lambda self, e: self.binary(e, "?"),
    521            exp.ParseJSON: lambda self, e: self.sql(exp.cast(e.this, exp.DataType.Type.JSON)),
    522            exp.JSONPathKey: json_path_key_only_name,
    523            exp.JSONPathRoot: lambda *_: "",
    524            exp.JSONPathSubscript: lambda self, e: self.json_path_part(e.this),
    525            exp.LastDay: no_last_day_sql,
    526            exp.LogicalOr: rename_func("BOOL_OR"),
    527            exp.LogicalAnd: rename_func("BOOL_AND"),
    528            exp.Max: max_or_greatest,
    529            exp.MapFromEntries: no_map_from_entries_sql,
    530            exp.Min: min_or_least,
    531            exp.Merge: merge_without_target_sql,
    532            exp.PartitionedByProperty: lambda self, e: f"PARTITION BY {self.sql(e, 'this')}",
    533            exp.PercentileCont: transforms.preprocess(
    534                [transforms.add_within_group_for_percentiles]
    535            ),
    536            exp.PercentileDisc: transforms.preprocess(
    537                [transforms.add_within_group_for_percentiles]
    538            ),
    539            exp.Pivot: no_pivot_sql,
    540            exp.Pow: lambda self, e: self.binary(e, "^"),
    541            exp.Rand: rename_func("RANDOM"),
    542            exp.RegexpLike: lambda self, e: self.binary(e, "~"),
    543            exp.RegexpILike: lambda self, e: self.binary(e, "~*"),
    544            exp.Select: transforms.preprocess(
    545                [
    546                    transforms.eliminate_semi_and_anti_joins,
    547                    transforms.eliminate_qualify,
    548                ]
    549            ),
    550            exp.SHA2: sha256_sql,
    551            exp.StrPosition: str_position_sql,
    552            exp.StrToDate: lambda self, e: self.func("TO_DATE", e.this, self.format_time(e)),
    553            exp.StrToTime: lambda self, e: self.func("TO_TIMESTAMP", e.this, self.format_time(e)),
    554            exp.StructExtract: struct_extract_sql,
    555            exp.Substring: _substring_sql,
    556            exp.TimeFromParts: rename_func("MAKE_TIME"),
    557            exp.TimestampFromParts: rename_func("MAKE_TIMESTAMP"),
    558            exp.TimestampTrunc: timestamptrunc_sql(zone=True),
    559            exp.TimeStrToTime: timestrtotime_sql,
    560            exp.TimeToStr: lambda self, e: self.func("TO_CHAR", e.this, self.format_time(e)),
    561            exp.ToChar: lambda self, e: self.function_fallback_sql(e),
    562            exp.Trim: trim_sql,
    563            exp.TryCast: no_trycast_sql,
    564            exp.TsOrDsAdd: _date_add_sql("+"),
    565            exp.TsOrDsDiff: _date_diff_sql,
    566            exp.UnixToTime: lambda self, e: self.func("TO_TIMESTAMP", e.this),
    567            exp.TimeToUnix: lambda self, e: self.func(
    568                "DATE_PART", exp.Literal.string("epoch"), e.this
    569            ),
    570            exp.VariancePop: rename_func("VAR_POP"),
    571            exp.Variance: rename_func("VAR_SAMP"),
    572            exp.Xor: bool_xor_sql,
    573            exp.UnixToTime: _unix_to_time_sql,
    574        }
    575        TRANSFORMS.pop(exp.CommentColumnConstraint)
    576
    577        PROPERTIES_LOCATION = {
    578            **generator.Generator.PROPERTIES_LOCATION,
    579            exp.PartitionedByProperty: exp.Properties.Location.POST_SCHEMA,
    580            exp.TransientProperty: exp.Properties.Location.UNSUPPORTED,
    581            exp.VolatileProperty: exp.Properties.Location.UNSUPPORTED,
    582        }
    583
    584        def schemacommentproperty_sql(self, expression: exp.SchemaCommentProperty) -> str:
    585            self.unsupported("Table comments are not supported in the CREATE statement")
    586            return ""
    587
    588        def commentcolumnconstraint_sql(self, expression: exp.CommentColumnConstraint) -> str:
    589            self.unsupported("Column comments are not supported in the CREATE statement")
    590            return ""
    591
    592        def unnest_sql(self, expression: exp.Unnest) -> str:
    593            if len(expression.expressions) == 1:
    594                from sqlglot.optimizer.annotate_types import annotate_types
    595
    596                this = annotate_types(expression.expressions[0])
    597                if this.is_type("array<json>"):
    598                    while isinstance(this, exp.Cast):
    599                        this = this.this
    600
    601                    arg = self.sql(exp.cast(this, exp.DataType.Type.JSON))
    602                    alias = self.sql(expression, "alias")
    603                    alias = f" AS {alias}" if alias else ""
    604
    605                    if expression.args.get("offset"):
    606                        self.unsupported("Unsupported JSON_ARRAY_ELEMENTS with offset")
    607
    608                    return f"JSON_ARRAY_ELEMENTS({arg}){alias}"
    609
    610            return super().unnest_sql(expression)
    611
    612        def bracket_sql(self, expression: exp.Bracket) -> str:
    613            """Forms like ARRAY[1, 2, 3][3] aren't allowed; we need to wrap the ARRAY."""
    614            if isinstance(expression.this, exp.Array):
    615                expression.set("this", exp.paren(expression.this, copy=False))
    616
    617            return super().bracket_sql(expression)
    618
    619        def matchagainst_sql(self, expression: exp.MatchAgainst) -> str:
    620            this = self.sql(expression, "this")
    621            expressions = [f"{self.sql(e)} @@ {this}" for e in expression.expressions]
    622            sql = " OR ".join(expressions)
    623            return f"({sql})" if len(expressions) > 1 else sql
    624
    625        def alterset_sql(self, expression: exp.AlterSet) -> str:
    626            exprs = self.expressions(expression, flat=True)
    627            exprs = f"({exprs})" if exprs else ""
    628
    629            access_method = self.sql(expression, "access_method")
    630            access_method = f"ACCESS METHOD {access_method}" if access_method else ""
    631            tablespace = self.sql(expression, "tablespace")
    632            tablespace = f"TABLESPACE {tablespace}" if tablespace else ""
    633            option = self.sql(expression, "option")
    634
    635            return f"SET {exprs}{access_method}{tablespace}{option}"
    636
    637        def datatype_sql(self, expression: exp.DataType) -> str:
    638            if expression.is_type(exp.DataType.Type.ARRAY):
    639                if expression.expressions:
    640                    values = self.expressions(expression, key="values", flat=True)
    641                    return f"{self.expressions(expression, flat=True)}[{values}]"
    642                return "ARRAY"
    643            return super().datatype_sql(expression)
    644
    645        def cast_sql(self, expression: exp.Cast, safe_prefix: t.Optional[str] = None) -> str:
    646            this = expression.this
    647
    648            # Postgres casts DIV() to decimal for transpilation but when roundtripping it's superfluous
    649            if isinstance(this, exp.IntDiv) and expression.to == exp.DataType.build("decimal"):
    650                return self.sql(this)
    651
    652            return super().cast_sql(expression, safe_prefix=safe_prefix)

Generator converts a given syntax tree to the corresponding SQL string.

###### Arguments:

- pretty: Whether to format the produced SQL string. Default: False.
- identify: Determines when an identifier should be quoted. Possible values are: False (default): Never quote, except in cases where it's mandatory by the dialect. True or 'always': Always quote. 'safe': Only quote identifiers that are case insensitive.
- normalize: Whether to normalize identifiers to lowercase. Default: False.
- pad: The pad size in a formatted string. For example, this affects the indentation of a projection in a query, relative to its nesting level. Default: 2.
- indent: The indentation size in a formatted string. For example, this affects the indentation of subqueries and filters under a `WHERE` clause. Default: 2.
- normalize_functions: How to normalize function names. Possible values are: "upper" or True (default): Convert names to uppercase. "lower": Convert names to lowercase. False: Disables function name normalization.
- unsupported_level: Determines the generator's behavior when it encounters unsupported expressions. Default ErrorLevel.WARN.
- max_unsupported: Maximum number of unsupported messages to include in a raised UnsupportedError. This is only relevant if unsupported_level is ErrorLevel.RAISE. Default: 3
- leading_comma: Whether the comma is leading or trailing in select expressions. This is only relevant when generating in pretty mode. Default: False
- max_text_width: The max number of characters in a segment before creating new lines in pretty mode. The default is on the smaller end because the length only represents a segment and not the true line length. Default: 80
- comments: Whether to preserve comments in the output SQL code. Default: True

SINGLE_STRING_INTERVAL = True
RENAME_TABLE_WITH_DB = False
LOCKING_READS_SUPPORTED = True
JOIN_HINTS = False
TABLE_HINTS = False
QUERY_HINTS = False
NVL2_SUPPORTED = False
PARAMETER_TOKEN = '$'
TABLESAMPLE_SIZE_IS_ROWS = False
TABLESAMPLE_SEED_KEYWORD = 'REPEATABLE'
SUPPORTS_SELECT_INTO = True
JSON_TYPE_REQUIRED_FOR_EXTRACTION = True
SUPPORTS_UNLOGGED_TABLES = True
LIKE_PROPERTY_INSIDE_SCHEMA = True
MULTI_ARG_DISTINCT = False
CAN_IMPLEMENT_ARRAY_ANY = True
COPY_HAS_INTO_KEYWORD = False
SUPPORTED_JSON_PATH_PARTS = {<class 'sqlglot.expressions.JSONPathSubscript'>, <class 'sqlglot.expressions.JSONPathRoot'>, <class 'sqlglot.expressions.JSONPathKey'>}
TYPE_MAPPING = {<Type.NCHAR: 'NCHAR'>: 'CHAR', <Type.NVARCHAR: 'NVARCHAR'>: 'VARCHAR', <Type.MEDIUMTEXT: 'MEDIUMTEXT'>: 'TEXT', <Type.LONGTEXT: 'LONGTEXT'>: 'TEXT', <Type.TINYTEXT: 'TINYTEXT'>: 'TEXT', <Type.MEDIUMBLOB: 'MEDIUMBLOB'>: 'BLOB', <Type.LONGBLOB: 'LONGBLOB'>: 'BLOB', <Type.TINYBLOB: 'TINYBLOB'>: 'BLOB', <Type.INET: 'INET'>: 'INET', <Type.ROWVERSION: 'ROWVERSION'>: 'BYTEA', <Type.TINYINT: 'TINYINT'>: 'SMALLINT', <Type.FLOAT: 'FLOAT'>: 'REAL', <Type.DOUBLE: 'DOUBLE'>: 'DOUBLE PRECISION', <Type.BINARY: 'BINARY'>: 'BYTEA', <Type.VARBINARY: 'VARBINARY'>: 'BYTEA', <Type.DATETIME: 'DATETIME'>: 'TIMESTAMP'}
TRANSFORMS = {<class 'sqlglot.expressions.JSONPathKey'>: <function json_path_key_only_name>, <class 'sqlglot.expressions.JSONPathRoot'>: <function Postgres.Generator.<lambda>>, <class 'sqlglot.expressions.JSONPathSubscript'>: <function Postgres.Generator.<lambda>>, <class 'sqlglot.expressions.AllowedValuesProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.AutoRefreshProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.BackupProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.CaseSpecificColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.CharacterSetColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.CharacterSetProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.ClusteredColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.CollateColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.ConnectByRoot'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.CopyGrantsProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.DateFormatColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.DefaultColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.DynamicProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.EncodeColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.EphemeralColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.ExcludeColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.ExecuteAsProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.ExternalProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.GlobalProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.HeapProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.IcebergProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.InheritsProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.InlineLengthColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.InputModelProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.IntervalSpan'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.JSONExtract'>: <function _json_extract_sql.<locals>._generate>, <class 'sqlglot.expressions.JSONExtractScalar'>: <function _json_extract_sql.<locals>._generate>, <class 'sqlglot.expressions.LanguageProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.LocationProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.LogProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.MaterializedProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.NonClusteredColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.NoPrimaryIndexProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.NotForReplicationColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.OnCommitProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.OnProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.OnUpdateColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.OutputModelProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.PathColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.ProjectionPolicyColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.RemoteWithConnectionModelProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.ReturnsProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.SampleProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.SecureProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.SetConfigProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.SetProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.SettingsProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.SharingProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.SqlReadWriteProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.SqlSecurityProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.StabilityProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.StrictProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.TemporaryProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.TagColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.TitleColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.Timestamp'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.ToMap'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.ToTableProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.TransformModelProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.TransientProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.UppercaseColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.UnloggedProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.VarMap'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.ViewAttributeProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.VolatileProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.WithJournalTableProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.WithSchemaBindingProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.WithOperator'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.AnyValue'>: <function any_value_to_max_sql>, <class 'sqlglot.expressions.Array'>: <function Postgres.Generator.<lambda>>, <class 'sqlglot.expressions.ArrayConcat'>: <function rename_func.<locals>.<lambda>>, <class 'sqlglot.expressions.ArrayContainsAll'>: <function Postgres.Generator.<lambda>>, <class 'sqlglot.expressions.ArrayOverlaps'>: <function Postgres.Generator.<lambda>>, <class 'sqlglot.expressions.ArrayFilter'>: <function filter_array_using_unnest>, <class 'sqlglot.expressions.ArraySize'>: <function Postgres.Generator.<lambda>>, <class 'sqlglot.expressions.BitwiseXor'>: <function Postgres.Generator.<lambda>>, <class 'sqlglot.expressions.ColumnDef'>: <function preprocess.<locals>._to_sql>, <class 'sqlglot.expressions.CurrentDate'>: <function no_paren_current_date_sql>, <class 'sqlglot.expressions.CurrentTimestamp'>: <function Postgres.Generator.<lambda>>, <class 'sqlglot.expressions.CurrentUser'>: <function Postgres.Generator.<lambda>>, <class 'sqlglot.expressions.DateAdd'>: <function _date_add_sql.<locals>.func>, <class 'sqlglot.expressions.DateDiff'>: <function _date_diff_sql>, <class 'sqlglot.expressions.DateStrToDate'>: <function datestrtodate_sql>, <class 'sqlglot.expressions.DateSub'>: <function _date_add_sql.<locals>.func>, <class 'sqlglot.expressions.Explode'>: <function rename_func.<locals>.<lambda>>, <class 'sqlglot.expressions.GroupConcat'>: <function _string_agg_sql>, <class 'sqlglot.expressions.IntDiv'>: <function rename_func.<locals>.<lambda>>, <class 'sqlglot.expressions.JSONBExtract'>: <function Postgres.Generator.<lambda>>, <class 'sqlglot.expressions.JSONBExtractScalar'>: <function Postgres.Generator.<lambda>>, <class 'sqlglot.expressions.JSONBContains'>: <function Postgres.Generator.<lambda>>, <class 'sqlglot.expressions.ParseJSON'>: <function Postgres.Generator.<lambda>>, <class 'sqlglot.expressions.LastDay'>: <function no_last_day_sql>, <class 'sqlglot.expressions.LogicalOr'>: <function rename_func.<locals>.<lambda>>, <class 'sqlglot.expressions.LogicalAnd'>: <function rename_func.<locals>.<lambda>>, <class 'sqlglot.expressions.Max'>: <function max_or_greatest>, <class 'sqlglot.expressions.MapFromEntries'>: <function no_map_from_entries_sql>, <class 'sqlglot.expressions.Min'>: <function min_or_least>, <class 'sqlglot.expressions.Merge'>: <function merge_without_target_sql>, <class 'sqlglot.expressions.PartitionedByProperty'>: <function Postgres.Generator.<lambda>>, <class 'sqlglot.expressions.PercentileCont'>: <function preprocess.<locals>._to_sql>, <class 'sqlglot.expressions.PercentileDisc'>: <function preprocess.<locals>._to_sql>, <class 'sqlglot.expressions.Pivot'>: <function no_pivot_sql>, <class 'sqlglot.expressions.Pow'>: <function Postgres.Generator.<lambda>>, <class 'sqlglot.expressions.Rand'>: <function rename_func.<locals>.<lambda>>, <class 'sqlglot.expressions.RegexpLike'>: <function Postgres.Generator.<lambda>>, <class 'sqlglot.expressions.RegexpILike'>: <function Postgres.Generator.<lambda>>, <class 'sqlglot.expressions.Select'>: <function preprocess.<locals>._to_sql>, <class 'sqlglot.expressions.SHA2'>: <function sha256_sql>, <class 'sqlglot.expressions.StrPosition'>: <function str_position_sql>, <class 'sqlglot.expressions.StrToDate'>: <function Postgres.Generator.<lambda>>, <class 'sqlglot.expressions.StrToTime'>: <function Postgres.Generator.<lambda>>, <class 'sqlglot.expressions.StructExtract'>: <function struct_extract_sql>, <class 'sqlglot.expressions.Substring'>: <function _substring_sql>, <class 'sqlglot.expressions.TimeFromParts'>: <function rename_func.<locals>.<lambda>>, <class 'sqlglot.expressions.TimestampFromParts'>: <function rename_func.<locals>.<lambda>>, <class 'sqlglot.expressions.TimestampTrunc'>: <function timestamptrunc_sql.<locals>._timestamptrunc_sql>, <class 'sqlglot.expressions.TimeStrToTime'>: <function timestrtotime_sql>, <class 'sqlglot.expressions.TimeToStr'>: <function Postgres.Generator.<lambda>>, <class 'sqlglot.expressions.ToChar'>: <function Postgres.Generator.<lambda>>, <class 'sqlglot.expressions.Trim'>: <function trim_sql>, <class 'sqlglot.expressions.TryCast'>: <function no_trycast_sql>, <class 'sqlglot.expressions.TsOrDsAdd'>: <function _date_add_sql.<locals>.func>, <class 'sqlglot.expressions.TsOrDsDiff'>: <function _date_diff_sql>, <class 'sqlglot.expressions.UnixToTime'>: <function _unix_to_time_sql>, <class 'sqlglot.expressions.TimeToUnix'>: <function Postgres.Generator.<lambda>>, <class 'sqlglot.expressions.VariancePop'>: <function rename_func.<locals>.<lambda>>, <class 'sqlglot.expressions.Variance'>: <function rename_func.<locals>.<lambda>>, <class 'sqlglot.expressions.Xor'>: <function bool_xor_sql>}
PROPERTIES_LOCATION = {<class 'sqlglot.expressions.AllowedValuesProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.AlgorithmProperty'>: <Location.POST_CREATE: 'POST_CREATE'>, <class 'sqlglot.expressions.AutoIncrementProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.AutoRefreshProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.BackupProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.BlockCompressionProperty'>: <Location.POST_NAME: 'POST_NAME'>, <class 'sqlglot.expressions.CharacterSetProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.ChecksumProperty'>: <Location.POST_NAME: 'POST_NAME'>, <class 'sqlglot.expressions.CollateProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.CopyGrantsProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.Cluster'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.ClusteredByProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.DataBlocksizeProperty'>: <Location.POST_NAME: 'POST_NAME'>, <class 'sqlglot.expressions.DataDeletionProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.DefinerProperty'>: <Location.POST_CREATE: 'POST_CREATE'>, <class 'sqlglot.expressions.DictRange'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.DictProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.DynamicProperty'>: <Location.POST_CREATE: 'POST_CREATE'>, <class 'sqlglot.expressions.DistKeyProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.DistStyleProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.EngineProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.ExecuteAsProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.ExternalProperty'>: <Location.POST_CREATE: 'POST_CREATE'>, <class 'sqlglot.expressions.FallbackProperty'>: <Location.POST_NAME: 'POST_NAME'>, <class 'sqlglot.expressions.FileFormatProperty'>: <Location.POST_WITH: 'POST_WITH'>, <class 'sqlglot.expressions.FreespaceProperty'>: <Location.POST_NAME: 'POST_NAME'>, <class 'sqlglot.expressions.GlobalProperty'>: <Location.POST_CREATE: 'POST_CREATE'>, <class 'sqlglot.expressions.HeapProperty'>: <Location.POST_WITH: 'POST_WITH'>, <class 'sqlglot.expressions.InheritsProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.IcebergProperty'>: <Location.POST_CREATE: 'POST_CREATE'>, <class 'sqlglot.expressions.InputModelProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.IsolatedLoadingProperty'>: <Location.POST_NAME: 'POST_NAME'>, <class 'sqlglot.expressions.JournalProperty'>: <Location.POST_NAME: 'POST_NAME'>, <class 'sqlglot.expressions.LanguageProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.LikeProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.LocationProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.LockProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.LockingProperty'>: <Location.POST_ALIAS: 'POST_ALIAS'>, <class 'sqlglot.expressions.LogProperty'>: <Location.POST_NAME: 'POST_NAME'>, <class 'sqlglot.expressions.MaterializedProperty'>: <Location.POST_CREATE: 'POST_CREATE'>, <class 'sqlglot.expressions.MergeBlockRatioProperty'>: <Location.POST_NAME: 'POST_NAME'>, <class 'sqlglot.expressions.NoPrimaryIndexProperty'>: <Location.POST_EXPRESSION: 'POST_EXPRESSION'>, <class 'sqlglot.expressions.OnProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.OnCommitProperty'>: <Location.POST_EXPRESSION: 'POST_EXPRESSION'>, <class 'sqlglot.expressions.Order'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.OutputModelProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.PartitionedByProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.PartitionedOfProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.PrimaryKey'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.Property'>: <Location.POST_WITH: 'POST_WITH'>, <class 'sqlglot.expressions.RemoteWithConnectionModelProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.ReturnsProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.RowFormatProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.RowFormatDelimitedProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.RowFormatSerdeProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.SampleProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.SchemaCommentProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.SecureProperty'>: <Location.POST_CREATE: 'POST_CREATE'>, <class 'sqlglot.expressions.SerdeProperties'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.Set'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.SettingsProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.SetProperty'>: <Location.POST_CREATE: 'POST_CREATE'>, <class 'sqlglot.expressions.SetConfigProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.SharingProperty'>: <Location.POST_EXPRESSION: 'POST_EXPRESSION'>, <class 'sqlglot.expressions.SequenceProperties'>: <Location.POST_EXPRESSION: 'POST_EXPRESSION'>, <class 'sqlglot.expressions.SortKeyProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.SqlReadWriteProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.SqlSecurityProperty'>: <Location.POST_CREATE: 'POST_CREATE'>, <class 'sqlglot.expressions.StabilityProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.StrictProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.TemporaryProperty'>: <Location.POST_CREATE: 'POST_CREATE'>, <class 'sqlglot.expressions.ToTableProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.TransientProperty'>: <Location.UNSUPPORTED: 'UNSUPPORTED'>, <class 'sqlglot.expressions.TransformModelProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.MergeTreeTTL'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.UnloggedProperty'>: <Location.POST_CREATE: 'POST_CREATE'>, <class 'sqlglot.expressions.ViewAttributeProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.VolatileProperty'>: <Location.UNSUPPORTED: 'UNSUPPORTED'>, <class 'sqlglot.expressions.WithDataProperty'>: <Location.POST_EXPRESSION: 'POST_EXPRESSION'>, <class 'sqlglot.expressions.WithJournalTableProperty'>: <Location.POST_NAME: 'POST_NAME'>, <class 'sqlglot.expressions.WithSchemaBindingProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.WithSystemVersioningProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>}
def schemacommentproperty_sql(self, expression: sqlglot.expressions.SchemaCommentProperty) -> str: View Source

    584        def schemacommentproperty_sql(self, expression: exp.SchemaCommentProperty) -> str:
    585            self.unsupported("Table comments are not supported in the CREATE statement")
    586            return ""

def commentcolumnconstraint_sql(self, expression: sqlglot.expressions.CommentColumnConstraint) -> str: View Source

    588        def commentcolumnconstraint_sql(self, expression: exp.CommentColumnConstraint) -> str:
    589            self.unsupported("Column comments are not supported in the CREATE statement")
    590            return ""

def unnest_sql(self, expression: sqlglot.expressions.Unnest) -> str: View Source

    592        def unnest_sql(self, expression: exp.Unnest) -> str:
    593            if len(expression.expressions) == 1:
    594                from sqlglot.optimizer.annotate_types import annotate_types
    595
    596                this = annotate_types(expression.expressions[0])
    597                if this.is_type("array<json>"):
    598                    while isinstance(this, exp.Cast):
    599                        this = this.this
    600
    601                    arg = self.sql(exp.cast(this, exp.DataType.Type.JSON))
    602                    alias = self.sql(expression, "alias")
    603                    alias = f" AS {alias}" if alias else ""
    604
    605                    if expression.args.get("offset"):
    606                        self.unsupported("Unsupported JSON_ARRAY_ELEMENTS with offset")
    607
    608                    return f"JSON_ARRAY_ELEMENTS({arg}){alias}"
    609
    610            return super().unnest_sql(expression)

def bracket_sql(self, expression: sqlglot.expressions.Bracket) -> str: View Source

    612        def bracket_sql(self, expression: exp.Bracket) -> str:
    613            """Forms like ARRAY[1, 2, 3][3] aren't allowed; we need to wrap the ARRAY."""
    614            if isinstance(expression.this, exp.Array):
    615                expression.set("this", exp.paren(expression.this, copy=False))
    616
    617            return super().bracket_sql(expression)

Forms like ARRAY[1, 2, 3][3] aren't allowed; we need to wrap the ARRAY.

def matchagainst_sql(self, expression: sqlglot.expressions.MatchAgainst) -> str: View Source

    619        def matchagainst_sql(self, expression: exp.MatchAgainst) -> str:
    620            this = self.sql(expression, "this")
    621            expressions = [f"{self.sql(e)} @@ {this}" for e in expression.expressions]
    622            sql = " OR ".join(expressions)
    623            return f"({sql})" if len(expressions) > 1 else sql

def alterset_sql(self, expression: sqlglot.expressions.AlterSet) -> str: View Source

    625        def alterset_sql(self, expression: exp.AlterSet) -> str:
    626            exprs = self.expressions(expression, flat=True)
    627            exprs = f"({exprs})" if exprs else ""
    628
    629            access_method = self.sql(expression, "access_method")
    630            access_method = f"ACCESS METHOD {access_method}" if access_method else ""
    631            tablespace = self.sql(expression, "tablespace")
    632            tablespace = f"TABLESPACE {tablespace}" if tablespace else ""
    633            option = self.sql(expression, "option")
    634
    635            return f"SET {exprs}{access_method}{tablespace}{option}"

def datatype_sql(self, expression: sqlglot.expressions.DataType) -> str: View Source

    637        def datatype_sql(self, expression: exp.DataType) -> str:
    638            if expression.is_type(exp.DataType.Type.ARRAY):
    639                if expression.expressions:
    640                    values = self.expressions(expression, key="values", flat=True)
    641                    return f"{self.expressions(expression, flat=True)}[{values}]"
    642                return "ARRAY"
    643            return super().datatype_sql(expression)

def cast_sql( self, expression: sqlglot.expressions.Cast, safe_prefix: Optional[str] = None) -> str: View Source

    645        def cast_sql(self, expression: exp.Cast, safe_prefix: t.Optional[str] = None) -> str:
    646            this = expression.this
    647
    648            # Postgres casts DIV() to decimal for transpilation but when roundtripping it's superfluous
    649            if isinstance(this, exp.IntDiv) and expression.to == exp.DataType.build("decimal"):
    650                return self.sql(this)
    651
    652            return super().cast_sql(expression, safe_prefix=safe_prefix)

SELECT_KINDS: Tuple[str, ...] = ()
TRY_SUPPORTED = False
SUPPORTS_UESCAPE = False
AFTER_HAVING_MODIFIER_TRANSFORMS = {'windows': <function Generator.<lambda>>, 'qualify': <function Generator.<lambda>>}

##### Inherited Members

sqlglot.generator.Generator
  Generator
  NULL_ORDERING_SUPPORTED
  IGNORE_NULLS_IN_FUNC
  EXPLICIT_SET_OP
  WRAP_DERIVED_VALUES
  CREATE_FUNCTION_RETURN_AS
  MATCHED_BY_SOURCE
  INTERVAL_ALLOWS_PLURAL_FORM
  LIMIT_FETCH
  LIMIT_ONLY_LITERALS
  GROUPINGS_SEP
  INDEX_ON
  QUERY_HINT_SEP
  IS_BOOL_ALLOWED
  DUPLICATE_KEY_UPDATE_WITH_SET
  LIMIT_IS_TOP
  RETURNING_END
  EXTRACT_ALLOWS_QUOTES
  TZ_TO_WITH_TIME_ZONE
  VALUES_AS_TABLE
  ALTER_TABLE_INCLUDE_COLUMN_KEYWORD
  UNNEST_WITH_ORDINALITY
  AGGREGATE_FILTER_SUPPORTED
  SEMI_ANTI_JOIN_WITH_SIDE
  COMPUTED_COLUMN_WITH_TYPE
  SUPPORTS_TABLE_COPY
  TABLESAMPLE_REQUIRES_PARENS
  TABLESAMPLE_KEYWORDS
  TABLESAMPLE_WITH_METHOD
  COLLATE_IS_FUNC
  DATA_TYPE_SPECIFIERS_ALLOWED
  ENSURE_BOOLS
  CTE_RECURSIVE_KEYWORD_REQUIRED
  SUPPORTS_SINGLE_ARG_CONCAT
  LAST_DAY_SUPPORTS_DATE_PART
  SUPPORTS_TABLE_ALIAS_COLUMNS
  UNPIVOT_ALIASES_ARE_IDENTIFIERS
  JSON_KEY_VALUE_PAIR_SEP
  INSERT_OVERWRITE
  SUPPORTS_CREATE_TABLE_LIKE
  JSON_PATH_BRACKETED_KEY_SUPPORTED
  JSON_PATH_SINGLE_QUOTE_ESCAPE
  SUPPORTS_TO_NUMBER
  SET_OP_MODIFIERS
  COPY_PARAMS_ARE_WRAPPED
  COPY_PARAMS_EQ_REQUIRED
  STAR_EXCEPT
  HEX_FUNC
  WITH_PROPERTIES_PREFIX
  QUOTE_JSON_PATH
  PAD_FILL_PATTERN_IS_REQUIRED
  PARSE_JSON_NAME
  TIME_PART_SINGULARS
  TOKEN_MAPPING
  STRUCT_DELIMITER
  NAMED_PLACEHOLDER_TOKEN
  RESERVED_KEYWORDS
  WITH_SEPARATED_COMMENTS
  EXCLUDE_COMMENTS
  UNWRAPPED_INTERVAL_VALUES
  PARAMETERIZABLE_TEXT_TYPES
  EXPRESSIONS_WITHOUT_NESTED_CTES
  SENTINEL_LINE_BREAK
  pretty
  identify
  normalize
  pad
  unsupported_level
  max_unsupported
  leading_comma
  max_text_width
  comments
  dialect
  normalize_functions
  unsupported_messages
  generate
  preprocess
  unsupported
  sep
  seg
  pad_comment
  maybe_comment
  wrap
  no_identify
  normalize_func
  indent
  sql
  uncache_sql
  cache_sql
  characterset_sql
  column_parts
  column_sql
  columnposition_sql
  columndef_sql
  columnconstraint_sql
  computedcolumnconstraint_sql
  autoincrementcolumnconstraint_sql
  compresscolumnconstraint_sql
  generatedasidentitycolumnconstraint_sql
  generatedasrowcolumnconstraint_sql
  periodforsystemtimeconstraint_sql
  notnullcolumnconstraint_sql
  transformcolumnconstraint_sql
  primarykeycolumnconstraint_sql
  uniquecolumnconstraint_sql
  createable_sql
  create_sql
  sequenceproperties_sql
  clone_sql
  describe_sql
  heredoc_sql
  prepend_ctes
  with_sql
  cte_sql
  tablealias_sql
  bitstring_sql
  hexstring_sql
  bytestring_sql
  unicodestring_sql
  rawstring_sql
  datatypeparam_sql
  directory_sql
  delete_sql
  drop_sql
  except_sql
  except_op
  fetch_sql
  filter_sql
  hint_sql
  indexparameters_sql
  index_sql
  identifier_sql
  hex_sql
  lowerhex_sql
  inputoutputformat_sql
  national_sql
  partition_sql
  properties_sql
  root_properties
  properties
  with_properties
  locate_properties
  property_name
  property_sql
  likeproperty_sql
  fallbackproperty_sql
  journalproperty_sql
  freespaceproperty_sql
  checksumproperty_sql
  mergeblockratioproperty_sql
  datablocksizeproperty_sql
  blockcompressionproperty_sql
  isolatedloadingproperty_sql
  partitionboundspec_sql
  partitionedofproperty_sql
  lockingproperty_sql
  withdataproperty_sql
  withsystemversioningproperty_sql
  insert_sql
  intersect_sql
  intersect_op
  introducer_sql
  kill_sql
  pseudotype_sql
  objectidentifier_sql
  onconflict_sql
  returning_sql
  rowformatdelimitedproperty_sql
  withtablehint_sql
  indextablehint_sql
  historicaldata_sql
  table_parts
  table_sql
  tablesample_sql
  pivot_sql
  version_sql
  tuple_sql
  update_sql
  values_sql
  var_sql
  into_sql
  from_sql
  group_sql
  having_sql
  connect_sql
  prior_sql
  join_sql
  lambda_sql
  lateral_op
  lateral_sql
  limit_sql
  offset_sql
  setitem_sql
  set_sql
  pragma_sql
  lock_sql
  literal_sql
  escape_str
  loaddata_sql
  null_sql
  boolean_sql
  order_sql
  withfill_sql
  cluster_sql
  distribute_sql
  sort_sql
  ordered_sql
  matchrecognizemeasure_sql
  matchrecognize_sql
  query_modifiers
  options_modifier
  queryoption_sql
  offset_limit_modifiers
  after_limit_modifiers
  select_sql
  schema_sql
  schema_columns_sql
  star_sql
  parameter_sql
  sessionparameter_sql
  placeholder_sql
  subquery_sql
  qualify_sql
  set_operations
  union_sql
  union_op
  prewhere_sql
  where_sql
  window_sql
  partition_by_sql
  windowspec_sql
  withingroup_sql
  between_sql
  bracket_offset_expressions
  all_sql
  any_sql
  exists_sql
  case_sql
  constraint_sql
  nextvaluefor_sql
  extract_sql
  trim_sql
  convert_concat_args
  concat_sql
  concatws_sql
  check_sql
  foreignkey_sql
  primarykey_sql
  if_sql
  jsonkeyvalue_sql
  jsonpath_sql
  json_path_part
  formatjson_sql
  jsonobject_sql
  jsonobjectagg_sql
  jsonarray_sql
  jsonarrayagg_sql
  jsoncolumndef_sql
  jsonschema_sql
  jsontable_sql
  openjsoncolumndef_sql
  openjson_sql
  in_sql
  in_unnest_op
  interval_sql
  return_sql
  reference_sql
  anonymous_sql
  paren_sql
  neg_sql
  not_sql
  alias_sql
  pivotalias_sql
  aliases_sql
  atindex_sql
  attimezone_sql
  fromtimezone_sql
  add_sql
  and_sql
  or_sql
  xor_sql
  connector_sql
  bitwiseand_sql
  bitwiseleftshift_sql
  bitwisenot_sql
  bitwiseor_sql
  bitwiserightshift_sql
  bitwisexor_sql
  currentdate_sql
  currenttimestamp_sql
  collate_sql
  command_sql
  comment_sql
  mergetreettlaction_sql
  mergetreettl_sql
  transaction_sql
  commit_sql
  rollback_sql
  altercolumn_sql
  alterdiststyle_sql
  altersortkey_sql
  renametable_sql
  renamecolumn_sql
  altertable_sql
  add_column_sql
  droppartition_sql
  addconstraint_sql
  distinct_sql
  ignorenulls_sql
  respectnulls_sql
  havingmax_sql
  intdiv_sql
  dpipe_sql
  div_sql
  overlaps_sql
  distance_sql
  dot_sql
  eq_sql
  propertyeq_sql
  escape_sql
  glob_sql
  gt_sql
  gte_sql
  ilike_sql
  ilikeany_sql
  is_sql
  like_sql
  likeany_sql
  similarto_sql
  lt_sql
  lte_sql
  mod_sql
  mul_sql
  neq_sql
  nullsafeeq_sql
  nullsafeneq_sql
  slice_sql
  sub_sql
  trycast_sql
  try_sql
  log_sql
  use_sql
  binary
  function_fallback_sql
  func
  format_args
  too_wide
  format_time
  expressions
  op_expressions
  naked_property
  tag_sql
  token_sql
  userdefinedfunction_sql
  joinhint_sql
  kwarg_sql
  when_sql
  merge_sql
  tochar_sql
  tonumber_sql
  dictproperty_sql
  dictrange_sql
  dictsubproperty_sql
  oncluster_sql
  clusteredbyproperty_sql
  anyvalue_sql
  querytransform_sql
  indexconstraintoption_sql
  checkcolumnconstraint_sql
  indexcolumnconstraint_sql
  nvl2_sql
  comprehension_sql
  columnprefix_sql
  opclass_sql
  predict_sql
  forin_sql
  refresh_sql
  operator_sql
  toarray_sql
  tsordstotime_sql
  tsordstotimestamp_sql
  tsordstodate_sql
  unixdate_sql
  lastday_sql
  dateadd_sql
  arrayany_sql
  generateseries_sql
  struct_sql
  partitionrange_sql
  truncatetable_sql
  convert_sql
  copyparameter_sql
  credentials_sql
  copy_sql
  semicolon_sql
  datadeletionproperty_sql
  maskingpolicycolumnconstraint_sql
  gapfill_sql
  scope_resolution
  scoperesolution_sql
  parsejson_sql
  length_sql
  rand_sql
  strtodate_sql
  strtotime_sql
  changes_sql
  pad_sql